# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8
# flake8: noqa
__doc__ = """
An API for working with Bladed Next Gen models.
"""
from dnv_bladed_models.models_impl import BladedEntity
from dnv_bladed_models.accelerometer import Accelerometer
from dnv_bladed_models.active_damper import ActiveDamper
from dnv_bladed_models.actuator_response import ActuatorResponse, ActuatorResponse_ActuatorResponseTypeEnum
from dnv_bladed_models.actuator_response_insert import ActuatorResponseInsert
from dnv_bladed_models.added_inertia import AddedInertia, AddedInertia_AddedInertiaTypeEnum
from dnv_bladed_models.added_inertia_insert import AddedInertiaInsert
from dnv_bladed_models.additional_constrained_wave import AdditionalConstrainedWave, AdditionalConstrainedWave_MethodEnum
from dnv_bladed_models.aerodynamic_information_calculation import AerodynamicInformationCalculation
from dnv_bladed_models.aerodynamic_model import AerodynamicModel, AerodynamicModel_AerodynamicModelTypeEnum
from dnv_bladed_models.aerodynamic_model_insert import AerodynamicModelInsert
from dnv_bladed_models.aerodynamic_properties import AerodynamicProperties
from dnv_bladed_models.aerodynamic_settings import AerodynamicSettings
from dnv_bladed_models.aerofoil import Aerofoil
from dnv_bladed_models.aerofoil_library import AerofoilLibrary
from dnv_bladed_models.aileron import Aileron
from dnv_bladed_models.aileron_aerofoil_library import AileronAerofoilLibrary
from dnv_bladed_models.air_characteristics import AirCharacteristics
from dnv_bladed_models.alpha_line import AlphaLine
from dnv_bladed_models.angle_range import AngleRange
from dnv_bladed_models.angle_vs_stiffness_torque import AngleVsStiffnessTorque
from dnv_bladed_models.applied_load import AppliedLoad, AppliedLoad_AppliedLoadTypeEnum
from dnv_bladed_models.applied_load_insert import AppliedLoadInsert
from dnv_bladed_models.applied_moment_vs_friction_torque import AppliedMomentVsFrictionTorque
from dnv_bladed_models.applied_xy_force_vs_friction_torque import AppliedXYForceVsFrictionTorque
from dnv_bladed_models.applied_z_force_vs_friction_torque import AppliedZForceVsFrictionTorque
from dnv_bladed_models.assembly import Assembly
from dnv_bladed_models.assembly_node import AssemblyNode
from dnv_bladed_models.automatic_mac_camy_fuchs import AutomaticMacCamyFuchs
from dnv_bladed_models.automatic_simple_cutoff_frequency import AutomaticSimpleCutoffFrequency
from dnv_bladed_models.beddoes_leishman_model import BeddoesLeishmanModel
from dnv_bladed_models.bending_torsion_coupling_terms_about_elastic_xy import BendingTorsionCouplingTermsAboutElasticXY
from dnv_bladed_models.blade import Blade
from dnv_bladed_models.blade_additional_inertia import BladeAdditionalInertia
from dnv_bladed_models.blade_aerodynamic_outputs import BladeAerodynamicOutputs
from dnv_bladed_models.blade_element_momentum import BladeElementMomentum
from dnv_bladed_models.blade_element_theory_corrections import BladeElementTheoryCorrections
from dnv_bladed_models.blade_hydrodynamic_outputs import BladeHydrodynamicOutputs
from dnv_bladed_models.blade_load_outputs import BladeLoadOutputs
from dnv_bladed_models.blade_modelling import BladeModelling, BladeModelling_BladeModellingTypeEnum, BladeModelling_GeometricStiffnessModelEnum
from dnv_bladed_models.blade_modelling_insert import BladeModellingInsert
from dnv_bladed_models.blade_motion_outputs import BladeMotionOutputs
from dnv_bladed_models.blade_output_group import BladeOutputGroup
from dnv_bladed_models.blade_output_group_library import BladeOutputGroupLibrary
from dnv_bladed_models.blade_outputs_for_cross_section import BladeOutputsForCrossSection
from dnv_bladed_models.blade_outputs_for_location import BladeOutputsForLocation, BladeOutputsForLocation_BladeOutputsForLocationTypeEnum
from dnv_bladed_models.blade_outputs_for_location_insert import BladeOutputsForLocationInsert
from dnv_bladed_models.blade_outputs_for_position import BladeOutputsForPosition
from dnv_bladed_models.blade_point_loading import BladePointLoading
from dnv_bladed_models.blade_point_mass import BladePointMass
from dnv_bladed_models.blade_stability_analysis import BladeStabilityAnalysis
from dnv_bladed_models.blade_stability_analysis_calculation import BladeStabilityAnalysisCalculation
from dnv_bladed_models.blade_stability_analysis_idling import BladeStabilityAnalysisIdling
from dnv_bladed_models.blade_stability_analysis_mode_of_operation import BladeStabilityAnalysisModeOfOperation, BladeStabilityAnalysisModeOfOperation_BladeStabilityAnalysisModeOfOperationTypeEnum
from dnv_bladed_models.blade_stability_analysis_mode_of_operation_insert import BladeStabilityAnalysisModeOfOperationInsert
from dnv_bladed_models.blade_stability_analysis_parked import BladeStabilityAnalysisParked
from dnv_bladed_models.blade_stability_perturbations import BladeStabilityPerturbations
from dnv_bladed_models.bladed_analysis import BladedAnalysis
from dnv_bladed_models.bladed_control import BladedControl
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.brake import Brake, Brake_BrakeTypeEnum, Brake_BrakePositionEnum
from dnv_bladed_models.brake_insert import BrakeInsert
from dnv_bladed_models.campbell_diagram import CampbellDiagram
from dnv_bladed_models.campbell_diagram_mode_of_operation import CampbellDiagramModeOfOperation, CampbellDiagramModeOfOperation_CampbellDiagramModeOfOperationTypeEnum
from dnv_bladed_models.campbell_diagram_mode_of_operation_insert import CampbellDiagramModeOfOperationInsert
from dnv_bladed_models.campbell_idling import CampbellIdling
from dnv_bladed_models.campbell_parked import CampbellParked
from dnv_bladed_models.campbell_power_production import CampbellPowerProduction
from dnv_bladed_models.circular_lidar_scan import CircularLidarScan
from dnv_bladed_models.component import Component, Component_ComponentTypeEnum
from dnv_bladed_models.component_insert import ComponentInsert
from dnv_bladed_models.component_library import ComponentLibrary
from dnv_bladed_models.compressible_beddoes_leishman_model import CompressibleBeddoesLeishmanModel
from dnv_bladed_models.conditional_event import ConditionalEvent
from dnv_bladed_models.connectable_node import ConnectableNode, ConnectableNode_ComponentTypesAllowedEnum
from dnv_bladed_models.constants import Constants
from dnv_bladed_models.controlled_operation import ControlledOperation
from dnv_bladed_models.controlled_stop import ControlledStop
from dnv_bladed_models.controller_fault import ControllerFault
from dnv_bladed_models.controller_lidar_settings import ControllerLidarSettings
from dnv_bladed_models.coordinate_systems import CoordinateSystems
from dnv_bladed_models.cross_section import CrossSection
from dnv_bladed_models.current import Current, Current_CurrentTypeEnum
from dnv_bladed_models.current_direction_shear_time_history import CurrentDirectionShearTimeHistory
from dnv_bladed_models.current_direction_shear_variation import CurrentDirectionShearVariation, CurrentDirectionShearVariation_DirectionShearVariationTypeEnum
from dnv_bladed_models.current_direction_shear_variation_insert import CurrentDirectionShearVariationInsert
from dnv_bladed_models.current_direction_time_history import CurrentDirectionTimeHistory
from dnv_bladed_models.current_direction_variation import CurrentDirectionVariation, CurrentDirectionVariation_DirectionVariationTypeEnum
from dnv_bladed_models.current_direction_variation_insert import CurrentDirectionVariationInsert
from dnv_bladed_models.current_horizontal_shear_time_history import CurrentHorizontalShearTimeHistory
from dnv_bladed_models.current_horizontal_shear_variation import CurrentHorizontalShearVariation, CurrentHorizontalShearVariation_HorizontalShearVariationTypeEnum
from dnv_bladed_models.current_horizontal_shear_variation_insert import CurrentHorizontalShearVariationInsert
from dnv_bladed_models.current_insert import CurrentInsert
from dnv_bladed_models.current_mean_speed_time_history import CurrentMeanSpeedTimeHistory
from dnv_bladed_models.current_mean_speed_variation import CurrentMeanSpeedVariation, CurrentMeanSpeedVariation_MeanSpeedVariationTypeEnum
from dnv_bladed_models.current_mean_speed_variation_insert import CurrentMeanSpeedVariationInsert
from dnv_bladed_models.current_vertical_shear_time_history import CurrentVerticalShearTimeHistory
from dnv_bladed_models.current_vertical_shear_variation import CurrentVerticalShearVariation, CurrentVerticalShearVariation_VerticalShearVariationTypeEnum
from dnv_bladed_models.current_vertical_shear_variation_insert import CurrentVerticalShearVariationInsert
from dnv_bladed_models.currents import Currents
from dnv_bladed_models.custom_linear_model_dll import CustomLinearModelDll
from dnv_bladed_models.damper import Damper
from dnv_bladed_models.damper_force_or_torque_vs_rate import DamperForceOrTorqueVsRate
from dnv_bladed_models.dimensionless_range import DimensionlessRange
from dnv_bladed_models.drivetrain_and_nacelle import DrivetrainAndNacelle
from dnv_bladed_models.drivetrain_and_nacelle_mass_properties import DrivetrainAndNacelleMassProperties
from dnv_bladed_models.drivetrain_and_nacelle_moment_of_inertia import DrivetrainAndNacelleMomentOfInertia
from dnv_bladed_models.drivetrain_connectable_nodes import DrivetrainConnectableNodes
from dnv_bladed_models.drivetrain_output_group import DrivetrainOutputGroup
from dnv_bladed_models.drivetrain_output_group_library import DrivetrainOutputGroupLibrary
from dnv_bladed_models.dynamic_stall import DynamicStall, DynamicStall_DynamicStallTypeEnum
from dnv_bladed_models.dynamic_stall_insert import DynamicStallInsert
from dnv_bladed_models.dynamic_upstream_wake import DynamicUpstreamWake
from dnv_bladed_models.dynamic_wake import DynamicWake, DynamicWake_DynamicWakeTypeEnum, DynamicWake_AreaAveragingMethodEnum
from dnv_bladed_models.dynamic_wake_insert import DynamicWakeInsert
from dnv_bladed_models.earthquake import Earthquake
from dnv_bladed_models.electrical_grid import ElectricalGrid
from dnv_bladed_models.electrical_losses import ElectricalLosses, ElectricalLosses_ElectricalLossesTypeEnum
from dnv_bladed_models.electrical_losses_insert import ElectricalLossesInsert
from dnv_bladed_models.emergency_stop_button import EmergencyStopButton
from dnv_bladed_models.emergency_stop_operation import EmergencyStopOperation
from dnv_bladed_models.environment import Environment
from dnv_bladed_models.equilibrium_state_solver_settings import EquilibriumStateSolverSettings
from dnv_bladed_models.equilibrium_wake_model import EquilibriumWakeModel
from dnv_bladed_models.event import Event, Event_EventTypeEnum
from dnv_bladed_models.event_insert import EventInsert
from dnv_bladed_models.evolving_turbulence import EvolvingTurbulence, EvolvingTurbulence_EvolvingTurbulenceTypeEnum
from dnv_bladed_models.evolving_turbulence_exponential import EvolvingTurbulenceExponential
from dnv_bladed_models.evolving_turbulence_insert import EvolvingTurbulenceInsert
from dnv_bladed_models.evolving_turbulence_kristensen import EvolvingTurbulenceKristensen
from dnv_bladed_models.explicit_newmark_beta_fixed_step import ExplicitNewmarkBetaFixedStep
from dnv_bladed_models.explicit_tower_can import ExplicitTowerCan
from dnv_bladed_models.exponential_shear_model import ExponentialShearModel
from dnv_bladed_models.extension_pieces import ExtensionPieces
from dnv_bladed_models.external_controller import ExternalController, ExternalController_CallingConventionEnum, ExternalController_TimeStepMultiplierEnum
from dnv_bladed_models.external_flow_source_for_wind import ExternalFlowSourceForWind
from dnv_bladed_models.external_module import ExternalModule
from dnv_bladed_models.external_module_component import ExternalModuleComponent
from dnv_bladed_models.external_module_component_connectable_nodes import ExternalModuleComponentConnectableNodes
from dnv_bladed_models.externally_stepped_simulation import ExternallySteppedSimulation
from dnv_bladed_models.fault import Fault
from dnv_bladed_models.finite_element_blade_modelling import FiniteElementBladeModelling
from dnv_bladed_models.first_order_actuator_response import FirstOrderActuatorResponse
from dnv_bladed_models.first_order_response import FirstOrderResponse
from dnv_bladed_models.first_order_transducer_response import FirstOrderTransducerResponse
from dnv_bladed_models.fixed_speed_active_damper import FixedSpeedActiveDamper
from dnv_bladed_models.fixed_step_integrator import FixedStepIntegrator
from dnv_bladed_models.flex_body import FlexBody
from dnv_bladed_models.flexibility import Flexibility
from dnv_bladed_models.flexible_joint import FlexibleJoint
from dnv_bladed_models.flow_obstruction import FlowObstruction
from dnv_bladed_models.foundation import Foundation, Foundation_FoundationTypeEnum
from dnv_bladed_models.foundation_insert import FoundationInsert
from dnv_bladed_models.free_flow_model import FreeFlowModel
from dnv_bladed_models.frequency_vs_spectral_density import FrequencyVsSpectralDensity
from dnv_bladed_models.friction import Friction
from dnv_bladed_models.friction_torque_vs_stiction_torque import FrictionTorqueVsStictionTorque
from dnv_bladed_models.frozen_wake_model import FrozenWakeModel
from dnv_bladed_models.full_load_operation import FullLoadOperation
from dnv_bladed_models.gl2010_icing import GL2010Icing
from dnv_bladed_models.gaussian import Gaussian
from dnv_bladed_models.gearbox_mounting_rolling import GearboxMountingRolling
from dnv_bladed_models.generalised_alpha_fixed_step import GeneralisedAlphaFixedStep
from dnv_bladed_models.generator import Generator
from dnv_bladed_models.generator_output_group import GeneratorOutputGroup
from dnv_bladed_models.generator_output_group_library import GeneratorOutputGroupLibrary
from dnv_bladed_models.generator_overpower import GeneratorOverpower
from dnv_bladed_models.generator_overspeed import GeneratorOverspeed
from dnv_bladed_models.generator_short_circuit import GeneratorShortCircuit
from dnv_bladed_models.generator_speed_vs_generator_torque import GeneratorSpeedVsGeneratorTorque
from dnv_bladed_models.glauert_skew_wake_model import GlauertSkewWakeModel
from dnv_bladed_models.grid_loss import GridLoss
from dnv_bladed_models.grid_loss_energy_sink import GridLossEnergySink
from dnv_bladed_models.hardware_tracking import HardwareTracking
from dnv_bladed_models.height_up_tower_vs_cd_cm import HeightUpTowerVsCdCm
from dnv_bladed_models.height_vs_shear_factor import HeightVsShearFactor
from dnv_bladed_models.high_speed_shaft_flexibility import HighSpeedShaftFlexibility
from dnv_bladed_models.hub import Hub, Hub_RotationDirectionEnum, Hub_UpwindOrDownwindEnum
from dnv_bladed_models.hub_connectable_nodes import HubConnectableNodes
from dnv_bladed_models.hub_mass_properties import HubMassProperties
from dnv_bladed_models.hub_output_group import HubOutputGroup
from dnv_bladed_models.hub_output_group_library import HubOutputGroupLibrary
from dnv_bladed_models.hydrodynamic_calculation import HydrodynamicCalculation, HydrodynamicCalculation_HydrostaticsMorisonMethodEnum
from dnv_bladed_models.iag_model import IAGModel
from dnv_bladed_models.iec4_icing import IEC4Icing
from dnv_bladed_models.iced_condition import IcedCondition
from dnv_bladed_models.idealised_pitch_actuator import IdealisedPitchActuator
from dnv_bladed_models.idealised_yaw_manoeuvre import IdealisedYawManoeuvre
from dnv_bladed_models.immediate_response import ImmediateResponse
from dnv_bladed_models.implicit_newmark_beta_fixed_step import ImplicitNewmarkBetaFixedStep
from dnv_bladed_models.inclinometer import Inclinometer
from dnv_bladed_models.incompressible_beddoes_leishman_model import IncompressibleBeddoesLeishmanModel
from dnv_bladed_models.independent_pitch_hub import IndependentPitchHub
from dnv_bladed_models.initial_azimuth_position import InitialAzimuthPosition
from dnv_bladed_models.initial_condition import InitialCondition, InitialCondition_InitialConditionTypeEnum
from dnv_bladed_models.initial_condition_insert import InitialConditionInsert
from dnv_bladed_models.initial_control_state import InitialControlState
from dnv_bladed_models.initial_floating_position import InitialFloatingPosition
from dnv_bladed_models.initial_pitch_position import InitialPitchPosition
from dnv_bladed_models.initial_position import InitialPosition
from dnv_bladed_models.initial_rotor_speed import InitialRotorSpeed
from dnv_bladed_models.initial_yaw_angle import InitialYawAngle
from dnv_bladed_models.input_power_vs_loss import InputPowerVsLoss
from dnv_bladed_models.instantaneous_actuator_response import InstantaneousActuatorResponse
from dnv_bladed_models.instantaneous_transducer_response import InstantaneousTransducerResponse
from dnv_bladed_models.integrator import Integrator, Integrator_IntegratorTypeEnum
from dnv_bladed_models.integrator_insert import IntegratorInsert
from dnv_bladed_models.interface_load_file import InterfaceLoadFile
from dnv_bladed_models.interpolated_aerofoil_library import InterpolatedAerofoilLibrary
from dnv_bladed_models.irregular_waves import IrregularWaves
from dnv_bladed_models.jonswap_pierson_moskowitz import JonswapPiersonMoskowitz
from dnv_bladed_models.laminar_flow_current import LaminarFlowCurrent
from dnv_bladed_models.laminar_flow_wind import LaminarFlowWind
from dnv_bladed_models.lidar import Lidar
from dnv_bladed_models.lidar_beam import LidarBeam
from dnv_bladed_models.lidar_controller_scan import LidarControllerScan
from dnv_bladed_models.lidar_focal_distance_control import LidarFocalDistanceControl, LidarFocalDistanceControl_FocalDistanceControlTypeEnum, LidarFocalDistanceControl_FocussingModeEnum
from dnv_bladed_models.lidar_focal_distance_control_insert import LidarFocalDistanceControlInsert
from dnv_bladed_models.lidar_mounting_position import LidarMountingPosition
from dnv_bladed_models.lidar_scanning_pattern import LidarScanningPattern, LidarScanningPattern_ScanningPatternTypeEnum
from dnv_bladed_models.lidar_scanning_pattern_insert import LidarScanningPatternInsert
from dnv_bladed_models.linear_airy import LinearAiry
from dnv_bladed_models.linear_electrical_losses import LinearElectricalLosses
from dnv_bladed_models.linear_foundation import LinearFoundation
from dnv_bladed_models.linear_passive_damper import LinearPassiveDamper
from dnv_bladed_models.linear_pitch_actuator import LinearPitchActuator
from dnv_bladed_models.linear_spring_damper import LinearSpringDamper
from dnv_bladed_models.linearisation_calculation import LinearisationCalculation
from dnv_bladed_models.load_bank import LoadBank
from dnv_bladed_models.load_sensor import LoadSensor
from dnv_bladed_models.logarithmic_shear_model import LogarithmicShearModel
from dnv_bladed_models.logging import Logging, Logging_ConsoleLoggingLevelEnum, Logging_MessageFileLoggingLevelEnum, Logging_SupportingFilesEnum, Logging_VisualisationEnum
from dnv_bladed_models.look_up_shear_model import LookUpShearModel
from dnv_bladed_models.look_up_table_element import LookUpTableElement
from dnv_bladed_models.low_speed_shaft import LowSpeedShaft
from dnv_bladed_models.mac_camy_fuchs import MacCamyFuchs
from dnv_bladed_models.mass_properties import MassProperties
from dnv_bladed_models.meandering_wake import MeanderingWake
from dnv_bladed_models.measured_signal_properties import MeasuredSignalProperties
from dnv_bladed_models.mechanical_losses import MechanicalLosses
from dnv_bladed_models.modal_analysis_calculation import ModalAnalysisCalculation
from dnv_bladed_models.modal_blade_modelling import ModalBladeModelling
from dnv_bladed_models.modal_structural_modelling import ModalStructuralModelling
from dnv_bladed_models.model_linearisation import ModelLinearisation
from dnv_bladed_models.model_linearisation_perturbations import ModelLinearisationPerturbations
from dnv_bladed_models.momentum_theory_corrections import MomentumTheoryCorrections, MomentumTheoryCorrections_GlauertCorrectionMethodForHighInductionEnum
from dnv_bladed_models.mounting import Mounting
from dnv_bladed_models.mounting_points import MountingPoints
from dnv_bladed_models.multiple_lidar_focal_distances import MultipleLidarFocalDistances
from dnv_bladed_models.nacelle_accelerometer import NacelleAccelerometer
from dnv_bladed_models.nacelle_cover import NacelleCover
from dnv_bladed_models.nacelle_inclinometer import NacelleInclinometer
from dnv_bladed_models.nacelle_sensor import NacelleSensor, NacelleSensor_NacelleSensorTypeEnum
from dnv_bladed_models.nacelle_sensor_insert import NacelleSensorInsert
from dnv_bladed_models.nacelle_vibration import NacelleVibration
from dnv_bladed_models.near_shore_current import NearShoreCurrent
from dnv_bladed_models.near_surface_current import NearSurfaceCurrent
from dnv_bladed_models.network_frequency_disturbance import NetworkFrequencyDisturbance
from dnv_bladed_models.network_voltage_disturbance import NetworkVoltageDisturbance
from dnv_bladed_models.nodes_at_heights import NodesAtHeights
from dnv_bladed_models.nominal_hub_wind_speed_vs_minimum_steady_state_pitch_angle import NominalHubWindSpeedVsMinimumSteadyStatePitchAngle
from dnv_bladed_models.nominal_hub_wind_speed_vs_rotor_speed_and_pitch_angle import NominalHubWindSpeedVsRotorSpeedAndPitchAngle
from dnv_bladed_models.non_linear_electrical_losses import NonLinearElectricalLosses
from dnv_bladed_models.non_linear_shaft_brake import NonLinearShaftBrake
from dnv_bladed_models.normal_stop_operation import NormalStopOperation
from dnv_bladed_models.offset_and_rotated_coordinate_system import OffsetAndRotatedCoordinateSystem
from dnv_bladed_models.outputs import Outputs, Outputs_FileFormatEnum
from dnv_bladed_models.oye_dynamic_wake import OyeDynamicWake
from dnv_bladed_models.oye_model import OyeModel
from dnv_bladed_models.pallet_mounting_flexibility import PalletMountingFlexibility
from dnv_bladed_models.pallet_nodding import PalletNodding
from dnv_bladed_models.pallet_rolling import PalletRolling
from dnv_bladed_models.partial_load_operation import PartialLoadOperation
from dnv_bladed_models.passive_transducer_response import PassiveTransducerResponse
from dnv_bladed_models.pendulum_damper import PendulumDamper
from dnv_bladed_models.pendulum_damper_mounting_position import PendulumDamperMountingPosition
from dnv_bladed_models.performance_coefficients_calculation import PerformanceCoefficientsCalculation
from dnv_bladed_models.permanently_stuck_pitch_system import PermanentlyStuckPitchSystem
from dnv_bladed_models.perturbation_settings import PerturbationSettings
from dnv_bladed_models.pitch_acceleration_limits import PitchAccelerationLimits
from dnv_bladed_models.pitch_actuation_limited import PitchActuationLimited
from dnv_bladed_models.pitch_actuator import PitchActuator, PitchActuator_ActuatorDriveTypeEnum
from dnv_bladed_models.pitch_actuator_insert import PitchActuatorInsert
from dnv_bladed_models.pitch_bearing_friction_rate_relationship import PitchBearingFrictionRateRelationship
from dnv_bladed_models.pitch_bearing_friction_xy_force_relationship import PitchBearingFrictionXYForceRelationship
from dnv_bladed_models.pitch_bearing_friction_xy_moment_relationship import PitchBearingFrictionXYMomentRelationship
from dnv_bladed_models.pitch_bearing_friction_z_force_relationship import PitchBearingFrictionZForceRelationship
from dnv_bladed_models.pitch_bearing_kinetic_friction import PitchBearingKineticFriction
from dnv_bladed_models.pitch_bearing_stiction import PitchBearingStiction
from dnv_bladed_models.pitch_constant_rate_safety_system import PitchConstantRateSafetySystem
from dnv_bladed_models.pitch_controller import PitchController
from dnv_bladed_models.pitch_controller_rate_safety_system import PitchControllerRateSafetySystem
from dnv_bladed_models.pitch_controller_torque_safety_system import PitchControllerTorqueSafetySystem
from dnv_bladed_models.pitch_end_stops import PitchEndStops
from dnv_bladed_models.pitch_fault import PitchFault
from dnv_bladed_models.pitch_fault_constant_rate import PitchFaultConstantRate
from dnv_bladed_models.pitch_fault_constant_torque import PitchFaultConstantTorque
from dnv_bladed_models.pitch_fault_limp import PitchFaultLimp
from dnv_bladed_models.pitch_fault_seizure import PitchFaultSeizure
from dnv_bladed_models.pitch_fault_seizure_at_angle import PitchFaultSeizureAtAngle
from dnv_bladed_models.pitch_force_limits import PitchForceLimits
from dnv_bladed_models.pitch_force_minimum_maximum import PitchForceMinimumMaximum
from dnv_bladed_models.pitch_limit_switches import PitchLimitSwitches
from dnv_bladed_models.pitch_position_demand import PitchPositionDemand
from dnv_bladed_models.pitch_position_vs_minimum_and_maximum_moment import PitchPositionVsMinimumAndMaximumMoment
from dnv_bladed_models.pitch_rate_demand import PitchRateDemand
from dnv_bladed_models.pitch_rate_demand_set_by_external_controller import PitchRateDemandSetByExternalController
from dnv_bladed_models.pitch_rate_limits import PitchRateLimits
from dnv_bladed_models.pitch_rate_varies_with_position import PitchRateVariesWithPosition
from dnv_bladed_models.pitch_rate_varies_with_time import PitchRateVariesWithTime
from dnv_bladed_models.pitch_rate_vs_minimum_and_maximum_moment import PitchRateVsMinimumAndMaximumMoment
from dnv_bladed_models.pitch_safety_limit_switches import PitchSafetyLimitSwitches
from dnv_bladed_models.pitch_safety_system import PitchSafetySystem, PitchSafetySystem_PitchSafetySystemTypeEnum
from dnv_bladed_models.pitch_safety_system_insert import PitchSafetySystemInsert
from dnv_bladed_models.pitch_system import PitchSystem
from dnv_bladed_models.pitch_system_connectable_nodes import PitchSystemConnectableNodes
from dnv_bladed_models.pitch_system_demand import PitchSystemDemand, PitchSystemDemand_PitchSystemDemandTypeEnum
from dnv_bladed_models.pitch_system_demand_insert import PitchSystemDemandInsert
from dnv_bladed_models.pitch_system_output_group import PitchSystemOutputGroup
from dnv_bladed_models.pitch_system_output_group_library import PitchSystemOutputGroupLibrary
from dnv_bladed_models.pitch_system_rotary_drive import PitchSystemRotaryDrive
from dnv_bladed_models.pitch_torque_limits import PitchTorqueLimits
from dnv_bladed_models.pitch_torque_minimum_maximum import PitchTorqueMinimumMaximum
from dnv_bladed_models.pitch_torque_set_by_external_controller import PitchTorqueSetByExternalController
from dnv_bladed_models.pitt_and_peters_model import PittAndPetersModel
from dnv_bladed_models.point_inertia import PointInertia
from dnv_bladed_models.point_on_span import PointOnSpan
from dnv_bladed_models.position_of_hub_centre import PositionOfHubCentre
from dnv_bladed_models.position_of_upwind_turbine import PositionOfUpwindTurbine
from dnv_bladed_models.position_proportional_integral_deriviative import PositionProportionalIntegralDeriviative, PositionProportionalIntegralDeriviative_DifferentialGainActionEnum
from dnv_bladed_models.preset_current_direction_shear_transient import PresetCurrentDirectionShearTransient, PresetCurrentDirectionShearTransient_VariationShapeEnum
from dnv_bladed_models.preset_current_direction_transient import PresetCurrentDirectionTransient, PresetCurrentDirectionTransient_VariationShapeEnum
from dnv_bladed_models.preset_current_horizontal_shear_transient import PresetCurrentHorizontalShearTransient, PresetCurrentHorizontalShearTransient_VariationShapeEnum
from dnv_bladed_models.preset_current_mean_speed_transient import PresetCurrentMeanSpeedTransient, PresetCurrentMeanSpeedTransient_VariationShapeEnum
from dnv_bladed_models.preset_current_vertical_shear_transient import PresetCurrentVerticalShearTransient, PresetCurrentVerticalShearTransient_VariationShapeEnum
from dnv_bladed_models.preset_lidar_scan import PresetLidarScan, PresetLidarScan_BeamSamplingEnum
from dnv_bladed_models.preset_wind_direction_shear_transient import PresetWindDirectionShearTransient, PresetWindDirectionShearTransient_VariationShapeEnum
from dnv_bladed_models.preset_wind_direction_transient import PresetWindDirectionTransient, PresetWindDirectionTransient_VariationShapeEnum
from dnv_bladed_models.preset_wind_horizontal_shear_transient import PresetWindHorizontalShearTransient, PresetWindHorizontalShearTransient_VariationShapeEnum
from dnv_bladed_models.preset_wind_mean_speed_transient import PresetWindMeanSpeedTransient, PresetWindMeanSpeedTransient_VariationShapeEnum
from dnv_bladed_models.preset_wind_vertical_shear_transient import PresetWindVerticalShearTransient, PresetWindVerticalShearTransient_VariationShapeEnum
from dnv_bladed_models.proportional_integral_deriviative import ProportionalIntegralDeriviative, ProportionalIntegralDeriviative_DifferentialGainActionEnum
from dnv_bladed_models.radius_vs_deficit import RadiusVsDeficit
from dnv_bladed_models.rate_proportional_integral_deriviative import RateProportionalIntegralDeriviative, RateProportionalIntegralDeriviative_DifferentialGainActionEnum
from dnv_bladed_models.rate_vs_friction_torque import RateVsFrictionTorque
from dnv_bladed_models.rate_vs_position import RateVsPosition
from dnv_bladed_models.rate_vs_time import RateVsTime
from dnv_bladed_models.reference_coordinate_system import ReferenceCoordinateSystem
from dnv_bladed_models.regular_waves import RegularWaves
from dnv_bladed_models.response_to_demand import ResponseToDemand, ResponseToDemand_ResponseToDemandTypeEnum
from dnv_bladed_models.response_to_demand_insert import ResponseToDemandInsert
from dnv_bladed_models.rigid_blade_modelling import RigidBladeModelling
from dnv_bladed_models.rigid_body_inertia import RigidBodyInertia
from dnv_bladed_models.rigid_body_point_inertia import RigidBodyPointInertia
from dnv_bladed_models.rigid_body_sixby_six_inertia import RigidBodySixbySixInertia
from dnv_bladed_models.rigid_structural_modelling import RigidStructuralModelling
from dnv_bladed_models.rosette_lidar_scan import RosetteLidarScan
from dnv_bladed_models.rotation import Rotation
from dnv_bladed_models.rotational_stiffness import RotationalStiffness
from dnv_bladed_models.rotor_idling_control_state import RotorIdlingControlState
from dnv_bladed_models.rotor_imbalances import RotorImbalances
from dnv_bladed_models.rotor_in_power_production import RotorInPowerProduction
from dnv_bladed_models.rotor_overspeed import RotorOverspeed
from dnv_bladed_models.rotor_parked_control_state import RotorParkedControlState
from dnv_bladed_models.runge_kutta_variable_step import RungeKuttaVariableStep
from dnv_bladed_models.safety_circuit_library import SafetyCircuitLibrary
from dnv_bladed_models.safety_system import SafetySystem
from dnv_bladed_models.safety_system_circuit import SafetySystemCircuit
from dnv_bladed_models.sea_state import SeaState
from dnv_bladed_models.second_order_actuator_response import SecondOrderActuatorResponse
from dnv_bladed_models.second_order_response import SecondOrderResponse
from dnv_bladed_models.second_order_transducer_response import SecondOrderTransducerResponse
from dnv_bladed_models.selected_component_output_group import SelectedComponentOutputGroup
from dnv_bladed_models.sensor import Sensor, Sensor_SensorTypeEnum
from dnv_bladed_models.sensor_insert import SensorInsert
from dnv_bladed_models.sequential_timestep_adaptation import SequentialTimestepAdaptation
from dnv_bladed_models.settings import Settings
from dnv_bladed_models.shaft_input_power_vs_power_loss import ShaftInputPowerVsPowerLoss
from dnv_bladed_models.shaft_input_torque_vs_resisting_torque import ShaftInputTorqueVsResistingTorque
from dnv_bladed_models.shaft_speed_vs_shaft_input_power_vs_power_loss import ShaftSpeedVsShaftInputPowerVsPowerLoss
from dnv_bladed_models.shaft_speed_vs_shaft_input_torque_vs_resisting_torque import ShaftSpeedVsShaftInputTorqueVsResistingTorque
from dnv_bladed_models.shear_stiffnesses_along_elastic_xy import ShearStiffnessesAlongElasticXY
from dnv_bladed_models.short_circuit import ShortCircuit
from dnv_bladed_models.signal_properties import SignalProperties, SignalProperties_SignalQualityEnum, SignalProperties_SignalNoiseEnum
from dnv_bladed_models.signal_properties_acceleration import SignalPropertiesAcceleration
from dnv_bladed_models.signal_properties_angle import SignalPropertiesAngle
from dnv_bladed_models.signal_properties_angular_acceleration import SignalPropertiesAngularAcceleration
from dnv_bladed_models.signal_properties_angular_velocity import SignalPropertiesAngularVelocity
from dnv_bladed_models.signal_properties_force import SignalPropertiesForce
from dnv_bladed_models.signal_properties_length import SignalPropertiesLength
from dnv_bladed_models.signal_properties_moment import SignalPropertiesMoment
from dnv_bladed_models.signal_properties_power import SignalPropertiesPower
from dnv_bladed_models.signal_properties_velocity import SignalPropertiesVelocity
from dnv_bladed_models.simple_cutoff_frequency import SimpleCutoffFrequency
from dnv_bladed_models.simple_shaft_brake import SimpleShaftBrake
from dnv_bladed_models.simple_tower_can import SimpleTowerCan
from dnv_bladed_models.simplified_linear_foundation import SimplifiedLinearFoundation
from dnv_bladed_models.single_lidar_focal_distance import SingleLidarFocalDistance
from dnv_bladed_models.six_by_six_inertia import SixBySixInertia
from dnv_bladed_models.skew_wake_model import SkewWakeModel
from dnv_bladed_models.slipping_clutch import SlippingClutch
from dnv_bladed_models.solver_settings import SolverSettings
from dnv_bladed_models.spinner import Spinner
from dnv_bladed_models.spring_force_or_torque_vs_position import SpringForceOrTorqueVsPosition
from dnv_bladed_models.standard_hub import StandardHub
from dnv_bladed_models.standard_pitch_limit_switches import StandardPitchLimitSwitches
from dnv_bladed_models.start_up_operation import StartUpOperation
from dnv_bladed_models.steady_calculation import SteadyCalculation, SteadyCalculation_SteadyCalculationTypeEnum
from dnv_bladed_models.steady_calculation_insert import SteadyCalculationInsert
from dnv_bladed_models.steady_calculation_outputs import SteadyCalculationOutputs
from dnv_bladed_models.steady_calculation_with_component_outputs import SteadyCalculationWithComponentOutputs
from dnv_bladed_models.steady_operational_loads_calculation import SteadyOperationalLoadsCalculation
from dnv_bladed_models.steady_parked_loads_calculation import SteadyParkedLoadsCalculation, SteadyParkedLoadsCalculation_SweepParameterEnum
from dnv_bladed_models.steady_power_curve_calculation import SteadyPowerCurveCalculation
from dnv_bladed_models.steady_wake_deficit import SteadyWakeDeficit, SteadyWakeDeficit_SteadyWakeDeficitTypeEnum
from dnv_bladed_models.steady_wake_deficit_insert import SteadyWakeDeficitInsert
from dnv_bladed_models.stiffness_properties import StiffnessProperties
from dnv_bladed_models.stream_function import StreamFunction
from dnv_bladed_models.structural_mode import StructuralMode
from dnv_bladed_models.structural_modelling import StructuralModelling, StructuralModelling_StructuralModellingTypeEnum, StructuralModelling_GeometricStiffnessModelEnum
from dnv_bladed_models.structural_modelling_insert import StructuralModellingInsert
from dnv_bladed_models.structure import Structure
from dnv_bladed_models.sub_surface_current import SubSurfaceCurrent
from dnv_bladed_models.superelement import Superelement, Superelement_WaveLoadingUnitsInFileEnum
from dnv_bladed_models.superelement_connectable_nodes import SuperelementConnectableNodes
from dnv_bladed_models.support_structure_point_loading import SupportStructurePointLoading
from dnv_bladed_models.tide import Tide
from dnv_bladed_models.time_domain_current import TimeDomainCurrent
from dnv_bladed_models.time_domain_outputs import TimeDomainOutputs
from dnv_bladed_models.time_domain_simulation import TimeDomainSimulation
from dnv_bladed_models.time_domain_wind import TimeDomainWind
from dnv_bladed_models.time_vs_current_direction import TimeVsCurrentDirection
from dnv_bladed_models.time_vs_current_direction_shear import TimeVsCurrentDirectionShear
from dnv_bladed_models.time_vs_current_horizontal_shear import TimeVsCurrentHorizontalShear
from dnv_bladed_models.time_vs_current_mean_speed import TimeVsCurrentMeanSpeed
from dnv_bladed_models.time_vs_current_vertical_shear import TimeVsCurrentVerticalShear
from dnv_bladed_models.time_vs_frequency_change import TimeVsFrequencyChange
from dnv_bladed_models.time_vs_torque import TimeVsTorque
from dnv_bladed_models.time_vs_voltage_change import TimeVsVoltageChange
from dnv_bladed_models.time_vs_wind_direction import TimeVsWindDirection
from dnv_bladed_models.time_vs_wind_direction_shear import TimeVsWindDirectionShear
from dnv_bladed_models.time_vs_wind_horizontal_shear import TimeVsWindHorizontalShear
from dnv_bladed_models.time_vs_wind_mean_speed import TimeVsWindMeanSpeed
from dnv_bladed_models.time_vs_wind_vertical_shear import TimeVsWindVerticalShear
from dnv_bladed_models.timed_event import TimedEvent
from dnv_bladed_models.tower import Tower
from dnv_bladed_models.tower_aerodynamic_properties import TowerAerodynamicProperties
from dnv_bladed_models.tower_can import TowerCan, TowerCan_TowerCanTypeEnum
from dnv_bladed_models.tower_can_cross_section_properties import TowerCanCrossSectionProperties
from dnv_bladed_models.tower_can_cross_section_properties_where_different import TowerCanCrossSectionPropertiesWhereDifferent
from dnv_bladed_models.tower_can_cross_section_with_sensors import TowerCanCrossSectionWithSensors
from dnv_bladed_models.tower_can_insert import TowerCanInsert
from dnv_bladed_models.tower_can_modelling import TowerCanModelling
from dnv_bladed_models.tower_can_properties_by_material import TowerCanPropertiesByMaterial
from dnv_bladed_models.tower_can_properties_by_material_where_different import TowerCanPropertiesByMaterialWhereDifferent
from dnv_bladed_models.tower_can_properties_explicit import TowerCanPropertiesExplicit
from dnv_bladed_models.tower_can_properties_explicit_where_different import TowerCanPropertiesExplicitWhereDifferent
from dnv_bladed_models.tower_can_torsional_properties import TowerCanTorsionalProperties
from dnv_bladed_models.tower_connectable_node import TowerConnectableNode
from dnv_bladed_models.tower_connectable_nodes import TowerConnectableNodes
from dnv_bladed_models.tower_hydrodynamic_properties import TowerHydrodynamicProperties
from dnv_bladed_models.tower_material import TowerMaterial
from dnv_bladed_models.tower_materials_library import TowerMaterialsLibrary
from dnv_bladed_models.tower_node_at_height_outputs import TowerNodeAtHeightOutputs
from dnv_bladed_models.tower_node_loads import TowerNodeLoads
from dnv_bladed_models.tower_node_outputs import TowerNodeOutputs
from dnv_bladed_models.tower_output_group import TowerOutputGroup, TowerOutputGroup_AxisSystemEnum
from dnv_bladed_models.tower_output_group_library import TowerOutputGroupLibrary
from dnv_bladed_models.tower_point_loading import TowerPointLoading
from dnv_bladed_models.tower_shadow_potential_flow import TowerShadowPotentialFlow
from dnv_bladed_models.tower_shadow_powles import TowerShadowPowles
from dnv_bladed_models.transducer_behaviour import TransducerBehaviour, TransducerBehaviour_TransducerBehaviourTypeEnum
from dnv_bladed_models.transducer_behaviour_insert import TransducerBehaviourInsert
from dnv_bladed_models.transducer_faults import TransducerFaults, TransducerFaults_FaultTypeEnum
from dnv_bladed_models.transducer_faults_acceleration import TransducerFaultsAcceleration
from dnv_bladed_models.transducer_faults_angle import TransducerFaultsAngle
from dnv_bladed_models.transducer_faults_angular_acceleration import TransducerFaultsAngularAcceleration
from dnv_bladed_models.transducer_faults_angular_velocity import TransducerFaultsAngularVelocity
from dnv_bladed_models.transducer_faults_force import TransducerFaultsForce
from dnv_bladed_models.transducer_faults_length import TransducerFaultsLength
from dnv_bladed_models.transducer_faults_moment import TransducerFaultsMoment
from dnv_bladed_models.transducer_faults_power import TransducerFaultsPower
from dnv_bladed_models.transducer_faults_velocity import TransducerFaultsVelocity
from dnv_bladed_models.translation import Translation
from dnv_bladed_models.translational_stiffness import TranslationalStiffness
from dnv_bladed_models.turbine import Turbine
from dnv_bladed_models.turbine_operational_parameters import TurbineOperationalParameters
from dnv_bladed_models.turbulent_current import TurbulentCurrent, TurbulentCurrent_CentreTurbulenceFileOnEnum
from dnv_bladed_models.turbulent_wind import TurbulentWind, TurbulentWind_CentreTurbulenceFileOnEnum, TurbulentWind_InterpolationMethodEnum
from dnv_bladed_models.use_setpoint_trajectory_planning import UseSetpointTrajectoryPlanning
from dnv_bladed_models.user_defined_wake_deficit import UserDefinedWakeDeficit
from dnv_bladed_models.user_defined_waves import UserDefinedWaves
from dnv_bladed_models.variable_speed_active_damper import VariableSpeedActiveDamper
from dnv_bladed_models.variable_speed_generator import VariableSpeedGenerator
from dnv_bladed_models.variable_speed_pitch_regulated_control_model import VariableSpeedPitchRegulatedControlModel
from dnv_bladed_models.vector3_d import Vector3D
from dnv_bladed_models.velocity_range import VelocityRange
from dnv_bladed_models.velocity_vs_damping_torque import VelocityVsDampingTorque
from dnv_bladed_models.vortex_line import VortexLine, VortexLine_WakeTypeEnum, VortexLine_CoreGrowthModelEnum, VortexLine_InitialVortexCoreSizeModelEnum
from dnv_bladed_models.wake_properties_of_upstream_turbine import WakePropertiesOfUpstreamTurbine
from dnv_bladed_models.water_characteristics import WaterCharacteristics
from dnv_bladed_models.wave_diffraction_approximation import WaveDiffractionApproximation, WaveDiffractionApproximation_WaveDiffractionApproximationTypeEnum
from dnv_bladed_models.wave_diffraction_approximation_insert import WaveDiffractionApproximationInsert
from dnv_bladed_models.wave_spectrum import WaveSpectrum, WaveSpectrum_WaveCurrentInteractionMethodEnum
from dnv_bladed_models.waves import Waves, Waves_WavesTypeEnum
from dnv_bladed_models.waves_insert import WavesInsert
from dnv_bladed_models.wind import Wind, Wind_WindTypeEnum
from dnv_bladed_models.wind_direction_shear_time_history import WindDirectionShearTimeHistory
from dnv_bladed_models.wind_direction_shear_variation import WindDirectionShearVariation, WindDirectionShearVariation_DirectionShearVariationTypeEnum
from dnv_bladed_models.wind_direction_shear_variation_insert import WindDirectionShearVariationInsert
from dnv_bladed_models.wind_direction_time_history import WindDirectionTimeHistory
from dnv_bladed_models.wind_direction_variation import WindDirectionVariation, WindDirectionVariation_DirectionVariationTypeEnum
from dnv_bladed_models.wind_direction_variation_insert import WindDirectionVariationInsert
from dnv_bladed_models.wind_horizontal_shear_time_history import WindHorizontalShearTimeHistory
from dnv_bladed_models.wind_horizontal_shear_variation import WindHorizontalShearVariation, WindHorizontalShearVariation_HorizontalShearVariationTypeEnum
from dnv_bladed_models.wind_horizontal_shear_variation_insert import WindHorizontalShearVariationInsert
from dnv_bladed_models.wind_insert import WindInsert
from dnv_bladed_models.wind_mean_speed_time_history import WindMeanSpeedTimeHistory
from dnv_bladed_models.wind_mean_speed_variation import WindMeanSpeedVariation, WindMeanSpeedVariation_MeanSpeedVariationTypeEnum
from dnv_bladed_models.wind_mean_speed_variation_insert import WindMeanSpeedVariationInsert
from dnv_bladed_models.wind_shear import WindShear, WindShear_WindShearTypeEnum
from dnv_bladed_models.wind_shear_insert import WindShearInsert
from dnv_bladed_models.wind_vertical_shear_time_history import WindVerticalShearTimeHistory
from dnv_bladed_models.wind_vertical_shear_variation import WindVerticalShearVariation, WindVerticalShearVariation_VerticalShearVariationTypeEnum
from dnv_bladed_models.wind_vertical_shear_variation_insert import WindVerticalShearVariationInsert
from dnv_bladed_models.xml_blade_outputs import XMLBladeOutputs
from dnv_bladed_models.yaw_actuation import YawActuation
from dnv_bladed_models.yaw_bearing import YawBearing
from dnv_bladed_models.yaw_fault import YawFault
from dnv_bladed_models.yaw_fault_constant_rate import YawFaultConstantRate
from dnv_bladed_models.yaw_fault_constant_torque import YawFaultConstantTorque
from dnv_bladed_models.yaw_fault_limp import YawFaultLimp
from dnv_bladed_models.yaw_manoeuvre import YawManoeuvre
from dnv_bladed_models.yaw_system import YawSystem
from dnv_bladed_models.yaw_system_connectable_nodes import YawSystemConnectableNodes
from dnv_bladed_models.yaw_system_output_group import YawSystemOutputGroup
from dnv_bladed_models.yaw_system_output_group_library import YawSystemOutputGroupLibrary



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.dynamic_stall import DynamicStall

from .schema_helper import SchemaHelper
from .models_impl import *


class IAGModel(DynamicStall):
    r"""
    The IAG dynamic stall model.
    
    Attributes
    ----------
    DynamicStallType : Literal['IAGModel'], default='IAGModel'
        Defines the specific type of DynamicStall model in use.  For a `IAGModel` object, this must always be set to a value of `IAGModel`.
    
    UseImpulsiveContributions : bool, default=True
        Enabling the impulsive contributions in lift and moment coefficient.
    
    PressureLagTimeConstant : float, default=1.7
        The lag between the pressure and the lift.
    
    VortexLiftTimeConstant : float, default=6
        The rate of decay of the vortex induced lift that is generated when aerofoils undergo a rapid change in angle of attack towards positive or negative stall.
    
    VortexTravelTimeConstant : float, default=6
        The time constant that controls the duration of the vortex induced lift that is generated when aerofoils undergo a rapid change in angle of attack towards positive or negative stall.
    
    AttachedFlowConstantA1 : float, default=0.3
        The constant A1 for the IAG dynamic stall model to control the attached flow states.
    
    AttachedFlowConstantA2 : float, default=0.7
        The constant A2 for the IAG dynamic stall model to control the attached flow states.
    
    AttachedFlowConstantB1 : float, default=0.7
        The constant B1 for the IAG dynamic stall model to control the attached flow states.
    
    AttachedFlowConstantB2 : float, default=0.53
        The constant B2 for the IAG dynamic stall model to control the attached flow states.
    
    Notes
    -----
    
    """
    DynamicStallType: Literal['IAGModel'] = Field(alias="DynamicStallType", default='IAGModel', allow_mutation=False, const=True) # type: ignore
    UseImpulsiveContributions: bool = Field(alias="UseImpulsiveContributions", default=None)
    PressureLagTimeConstant: float = Field(alias="PressureLagTimeConstant", default=None)
    VortexLiftTimeConstant: float = Field(alias="VortexLiftTimeConstant", default=None)
    VortexTravelTimeConstant: float = Field(alias="VortexTravelTimeConstant", default=None)
    AttachedFlowConstantA1: float = Field(alias="AttachedFlowConstantA1", default=None)
    AttachedFlowConstantA2: float = Field(alias="AttachedFlowConstantA2", default=None)
    AttachedFlowConstantB1: float = Field(alias="AttachedFlowConstantB1", default=None)
    AttachedFlowConstantB2: float = Field(alias="AttachedFlowConstantB2", default=None)

    _relative_schema_path = 'Settings/AerodynamicSettings/DynamicStall/IAGModel.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'DynamicStallType').merge(DynamicStall._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


IAGModel.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.angle_vs_stiffness_torque import AngleVsStiffnessTorque
from dnv_bladed_models.damper import Damper
from dnv_bladed_models.pendulum_damper_mounting_position import PendulumDamperMountingPosition
from dnv_bladed_models.velocity_vs_damping_torque import VelocityVsDampingTorque

from .schema_helper import SchemaHelper
from .models_impl import *


class PendulumDamper(Damper):
    r"""
    A \"pendulum\" or \"tuned mass\" damper, which uses a suspended mass to damp oscillations in a tower.
    
    Not supported yet.
    
    Attributes
    ----------
    ComponentType : Literal['PendulumDamper'], default='PendulumDamper', Not supported yet
        Defines the specific type of Component model in use.  For a `PendulumDamper` object, this must always be set to a value of `PendulumDamper`.
    
    Length : float, Not supported yet
        The length of the (rigid) pendulum arm.
    
    Mass : float, Not supported yet
        The mass suspended at the end of the pendulum arm.
    
    Inertia : float, Not supported yet
        Any added inertia for the pendulum.
    
    Stiffness : float, Not supported yet
        The constant stiffness term for the hinge of the pendulum.  This is in addition to the non-linear terms defined in the AngleVsStiffnessTorque parameter.
    
    Damping : float, Not supported yet
        The constant damping term for the hinge of the pendulum.  This is in addition to the non-linear terms defined in the VelocityVsDampingTorque parameter.
    
    InitialAngle : float, Not supported yet
        The initial angle of the pendulum at the beginning of the simulation.
    
    ConstantFriction : float, Not supported yet
        The constant friction torque applied to rotational hinge.  Any other friction contributions will be in addition to this.
    
    AngleVsStiffnessTorque : List[AngleVsStiffnessTorque], Not supported yet
        A look-up table of additional stiffnesses that vary with the pendulum's position.
    
    VelocityVsDampingTorque : List[VelocityVsDampingTorque], Not supported yet
        A look-up table of additional damping that vary with the pendulum's velocity.
    
    MountingPosition : PendulumDamperMountingPosition, Not supported yet
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    ComponentType: Literal['PendulumDamper'] = Field(alias="ComponentType", default='PendulumDamper', allow_mutation=False, const=True) # Not supported yet # type: ignore
    Length: float = Field(alias="Length", default=None) # Not supported yet
    Mass: float = Field(alias="Mass", default=None) # Not supported yet
    Inertia: float = Field(alias="Inertia", default=None) # Not supported yet
    Stiffness: float = Field(alias="Stiffness", default=None) # Not supported yet
    Damping: float = Field(alias="Damping", default=None) # Not supported yet
    InitialAngle: float = Field(alias="InitialAngle", default=None) # Not supported yet
    ConstantFriction: float = Field(alias="ConstantFriction", default=None) # Not supported yet
    AngleVsStiffnessTorque: List[AngleVsStiffnessTorque] = Field(alias="AngleVsStiffnessTorque", default=list()) # Not supported yet
    VelocityVsDampingTorque: List[VelocityVsDampingTorque] = Field(alias="VelocityVsDampingTorque", default=list()) # Not supported yet
    MountingPosition: PendulumDamperMountingPosition = Field(alias="MountingPosition", default=None) # Not supported yet

    _relative_schema_path = 'Components/Damper/PendulumDamper.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['AngleVsStiffnessTorque','VelocityVsDampingTorque',]),
        'ComponentType').merge(Damper._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


PendulumDamper.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.earthquake import Earthquake
from dnv_bladed_models.external_flow_source_for_wind import ExternalFlowSourceForWind
from dnv_bladed_models.laminar_flow_wind import LaminarFlowWind
from dnv_bladed_models.sea_state import SeaState
from dnv_bladed_models.turbulent_wind import TurbulentWind
from dnv_bladed_models.wind import Wind
from dnv_bladed_models.wind_insert import WindInsert

from .schema_helper import SchemaHelper
from .models_impl import *

TWindOptions = TypeVar('TWindOptions', ExternalFlowSourceForWind, WindInsert, LaminarFlowWind, TurbulentWind, Wind, )

class Environment(BladedModel):
    r"""
    The definition of the environment conditions affecting the turbine location during this simulation.
    
    Attributes
    ----------
    Wind : Union[ExternalFlowSourceForWind, WindInsert, LaminarFlowWind, TurbulentWind]
    
    SeaState : SeaState, Not supported yet
    
    Earthquake : Earthquake, Not supported yet
    
    Notes
    -----
    
    """
    Wind: Union[ExternalFlowSourceForWind, WindInsert, LaminarFlowWind, TurbulentWind] = Field(alias="Wind", default=None, discriminator='WindType')
    SeaState: SeaState = Field(alias="SeaState", default=None) # Not supported yet
    Earthquake: Earthquake = Field(alias="Earthquake", default=None) # Not supported yet

    _relative_schema_path = 'TimeDomainSimulation/Environment/Environment.json'
    _type_info = TypeInfo(
        set([('Wind', 'WindType'),]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def Wind_as_ExternalFlowSourceForWind(self) -> ExternalFlowSourceForWind:
        """
        Retrieves the value of Wind guaranteeing it is a ExternalFlowSourceForWind; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        ExternalFlowSourceForWind
            A model object, guaranteed to be a ExternalFlowSourceForWind.

        Raises
        ------
        TypeError
            If the value is not a ExternalFlowSourceForWind.
        """
        return self.Wind_as(ExternalFlowSourceForWind)


    @property
    def Wind_as_LaminarFlowWind(self) -> LaminarFlowWind:
        """
        Retrieves the value of Wind guaranteeing it is a LaminarFlowWind; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        LaminarFlowWind
            A model object, guaranteed to be a LaminarFlowWind.

        Raises
        ------
        TypeError
            If the value is not a LaminarFlowWind.
        """
        return self.Wind_as(LaminarFlowWind)


    @property
    def Wind_as_TurbulentWind(self) -> TurbulentWind:
        """
        Retrieves the value of Wind guaranteeing it is a TurbulentWind; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        TurbulentWind
            A model object, guaranteed to be a TurbulentWind.

        Raises
        ------
        TypeError
            If the value is not a TurbulentWind.
        """
        return self.Wind_as(TurbulentWind)


    @property
    def Wind_as_inline(self) -> Union[ExternalFlowSourceForWind, LaminarFlowWind, TurbulentWind]:
        """
        Retrieves the value of Wind as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[ExternalFlowSourceForWind, LaminarFlowWind, TurbulentWind]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of Wind; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.Wind, WindInsert) or self.Wind.is_insert:
            raise TypeError(f"Expected Wind value to be an in-line object, but it is currently in the '$insert' state.")
        return self.Wind


    def Wind_as(self, cls: Type[TWindOptions])-> TWindOptions:
        """
        Retrieves the value of Wind, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of Wind, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[ExternalFlowSourceForWind, WindInsert, LaminarFlowWind, TurbulentWind]]
            One of the valid concrete types of Wind, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TWindOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of Wind:
        >>> val_obj = model_obj.Wind_as(models.ExternalFlowSourceForWind)
        >>> val_obj = model_obj.Wind_as(models.LaminarFlowWind)
        >>> val_obj = model_obj.Wind_as(models.TurbulentWind)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.Wind_as(models.WindInsert)
        """
        if not isinstance(self.Wind, cls):
            raise TypeError(f"Expected Wind of type '{cls.__name__}' but was type '{type(self.Wind).__name__}'")
        return self.Wind


    def _entity(self) -> bool:
        return True


Environment.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.initial_control_state import InitialControlState

from .schema_helper import SchemaHelper
from .models_impl import *


class RotorInPowerProduction(InitialControlState):
    r"""
    The rotor begins the simulation in the power production state.  This is the default behaviour, and this initial condition is not required to be present, although it can be added to make it explicit.
    
    Attributes
    ----------
    InitialConditionType : Literal['RotorInPowerProduction'], default='RotorInPowerProduction'
        Defines the specific type of InitialCondition model in use.  For a `RotorInPowerProduction` object, this must always be set to a value of `RotorInPowerProduction`.
    
    Notes
    -----
    
    """
    InitialConditionType: Literal['RotorInPowerProduction'] = Field(alias="InitialConditionType", default='RotorInPowerProduction', allow_mutation=False, const=True) # type: ignore

    _relative_schema_path = 'TimeDomainSimulation/InitialCondition/RotorInPowerProduction.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'InitialConditionType').merge(InitialControlState._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


RotorInPowerProduction.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.blade_modelling import BladeModelling
from dnv_bladed_models.structural_mode import StructuralMode

from .schema_helper import SchemaHelper
from .models_impl import *


class FiniteElementBladeModelling(BladeModelling):
    r"""
    The properties for the structural modelling of the blade using finite element analysis without modal reduction.
    
    Attributes
    ----------
    BladeModellingType : Literal['FiniteElementBladeModelling'], default='FiniteElementBladeModelling'
        Defines the specific type of BladeModelling model in use.  For a `FiniteElementBladeModelling` object, this must always be set to a value of `FiniteElementBladeModelling`.
    
    WholeBladeModeDampingRatios : List[StructuralMode]
        List of known whole-blade mode damping ratios. The whole-blade damping ratios will be converted into finite element degrees of freedom by Bladed. If the list is incomplete, meaning it lacks damping ratios for all modes, Bladed will assign frequency-proportional damping based on the damping ratio of the highest defined mode. The list must contain at least one entry.
    
    Notes
    -----
    
    """
    BladeModellingType: Literal['FiniteElementBladeModelling'] = Field(alias="BladeModellingType", default='FiniteElementBladeModelling', allow_mutation=False, const=True) # type: ignore
    WholeBladeModeDampingRatios: List[StructuralMode] = Field(alias="WholeBladeModeDampingRatios", default=list())

    _relative_schema_path = 'Components/Blade/BladeModelling/FiniteElementBladeModelling.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['WholeBladeModeDampingRatios',]),
        'BladeModellingType').merge(BladeModelling._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


FiniteElementBladeModelling.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel

from .schema_helper import SchemaHelper
from .models_impl import *


class BladeElementTheoryCorrections(BladedModel):
    r"""
    Properties related to the blade element aspects of the Blade Element Momentum model.
    
    Attributes
    ----------
    UsePrandtlCorrectionForTipLoss : bool, default=True
        If true, a Prandtl correction for tip loss effects will be applied.
    
    UsePrandtlCorrectionForRootLoss : bool, default=False
        If true, a Prandtl correction for hub loss effects will be applied.
    
    UseDragInInduction : bool, default=True
        If true, the drag coefficient will be included in the induction calculations. This mainly influences the BEM solution near the root of the blade, where the drag force is significant.
    
    Notes
    -----
    
    """
    UsePrandtlCorrectionForTipLoss: bool = Field(alias="UsePrandtlCorrectionForTipLoss", default=None)
    UsePrandtlCorrectionForRootLoss: bool = Field(alias="UsePrandtlCorrectionForRootLoss", default=None)
    UseDragInInduction: bool = Field(alias="UseDragInInduction", default=None)

    _relative_schema_path = 'Settings/AerodynamicSettings/AerodynamicModel/BladeElementTheoryCorrections/BladeElementTheoryCorrections.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


BladeElementTheoryCorrections.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.accelerometer import Accelerometer
from dnv_bladed_models.aerodynamic_properties import AerodynamicProperties
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.coordinate_systems import CoordinateSystems
from dnv_bladed_models.inclinometer import Inclinometer
from dnv_bladed_models.load_sensor import LoadSensor
from dnv_bladed_models.mass_properties import MassProperties
from dnv_bladed_models.sensor import Sensor
from dnv_bladed_models.sensor_insert import SensorInsert
from dnv_bladed_models.stiffness_properties import StiffnessProperties

from .schema_helper import SchemaHelper
from .models_impl import *

TSensorOptions = TypeVar('TSensorOptions', Accelerometer, Inclinometer, SensorInsert, LoadSensor, Sensor, )

class CrossSection(BladedModel):
    r"""
    A blade cross-section which defines the geometric, structural, and aerodynamic properties of the blade.
    
    Attributes
    ----------
    IsCrossSectionAMultipartBoundary : bool, default=False
        If true, this cross-section will serve as a boundary between parts of a multipart blade, allowing the blade to be split accordingly. Note that it is not possible to split the blade at the first or last cross-section of the blade.
    
    CoordinateSystems : CoordinateSystems
    
    StiffnessProperties : StiffnessProperties
    
    AerodynamicProperties : AerodynamicProperties
    
    MassProperties : MassProperties
    
    Sensors : List[Union[Accelerometer, Inclinometer, SensorInsert, LoadSensor]]
        A list of sensors that the external controller has access to.  The indices of the sensors will depend on their type (LoadSensor, Accelerometer, or Inclinometer) and the order in which they are defined on the Blade, from inboard to outpurd.  The details of exactly where the sensors are will be unavailable if the component is encrypted.
    
    Notes
    -----
    
    """
    IsCrossSectionAMultipartBoundary: bool = Field(alias="IsCrossSectionAMultipartBoundary", default=None)
    CoordinateSystems: CoordinateSystems = Field(alias="CoordinateSystems", default=None)
    StiffnessProperties: StiffnessProperties = Field(alias="StiffnessProperties", default=None)
    AerodynamicProperties: AerodynamicProperties = Field(alias="AerodynamicProperties", default=None)
    MassProperties: MassProperties = Field(alias="MassProperties", default=None)
    Sensors: List[Annotated[Union[Accelerometer, Inclinometer, SensorInsert, LoadSensor], Field(discriminator='SensorType')]] = Field(alias="Sensors", default=list())

    _relative_schema_path = 'Components/Blade/CrossSection/CrossSection.json'
    _type_info = TypeInfo(
        set([]),
        set([('Sensors', 'SensorType'),]),
        set(['Sensors',]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def Sensors_as_inline(self) -> Iterable[Union[Accelerometer, Inclinometer, LoadSensor]]:
        """
        Retrieves the value of Sensors as an iterable of model objects; if any of the values are specified with a '$insert', an error is raised.

        Returns
        -------
        Iterable[Union[Accelerometer, Inclinometer, LoadSensor]]
            A list of model objects, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If any of the values are not one of the concrete types of Sensor; i.e. they are specified with an 'insert'.

        Examples
        --------
        >>> for i, Sensor_obj in enumerate(obj.Sensors_as_any):
                print(f"Element {i} is of type Sensor_obj.SensorType}")

        or

        >>> for Sensor_obj in obj.Sensors_as_any:
                # process object
        """
        for val in self.Sensors:
            if isinstance(val, SensorInsert) or val.is_insert == True:
                raise TypeError(f"Expected element value to be an in-line object, but it is currently in the '$insert' state.")
            yield val


    def Sensors_element_as_Accelerometer(self, index: int) -> Accelerometer:
        """
        Retrieves an object from the Sensors array field, ensuring it is a Accelerometer.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        Accelerometer
            A model object at the specified index, guaranteed to be a Accelerometer.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a Accelerometer.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a Accelerometer, that is specified in-line:
        >>> entry_obj = obj.Sensors_element_as_Accelerometer(2)
        """
        return self.Sensors_element_as(index, Accelerometer)


    def Sensors_element_as_Inclinometer(self, index: int) -> Inclinometer:
        """
        Retrieves an object from the Sensors array field, ensuring it is a Inclinometer.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        Inclinometer
            A model object at the specified index, guaranteed to be a Inclinometer.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a Inclinometer.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a Inclinometer, that is specified in-line:
        >>> entry_obj = obj.Sensors_element_as_Inclinometer(2)
        """
        return self.Sensors_element_as(index, Inclinometer)


    def Sensors_element_as_LoadSensor(self, index: int) -> LoadSensor:
        """
        Retrieves an object from the Sensors array field, ensuring it is a LoadSensor.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        LoadSensor
            A model object at the specified index, guaranteed to be a LoadSensor.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a LoadSensor.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a LoadSensor, that is specified in-line:
        >>> entry_obj = obj.Sensors_element_as_LoadSensor(2)
        """
        return self.Sensors_element_as(index, LoadSensor)


    def Sensors_element_as_inline(self, index: int) -> Union[Accelerometer, Inclinometer, LoadSensor]:
        """
        Retrieves an object from the Sensors array field; if the value is specified with a '$insert', an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        Union[Accelerometer, Inclinometer, LoadSensor]
            A model object at the specified index, guaranteed to not be an '$insert'.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not one of the concrete types of Sensor; i.e. it is specified with an '$insert'.

        Examples
        --------
        Get a reference to the 2nd element, as any of the concrete types:
        >>> Sensors_obj = obj.Sensors_element_as_inline(2)
        """
        val = self.Sensors[index]
        if isinstance(val, SensorInsert) or val.is_insert == True:
            raise TypeError(f"Expected element value at '{index}' to be an in-line object, but it is currently in the '$insert' state.")
        return val


    def Sensors_element_as(self, index: int, element_cls: Type[TSensorOptions]) -> TSensorOptions:
        """
        Retrieves an object from the Sensors array field, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of Sensor, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        element_cls: Type[Union[Accelerometer, Inclinometer, SensorInsert, LoadSensor]]
            One of the valid concrete types of Sensor, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TSensorOptions
            A model object of the specified type.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the model object for the specified index is not of the specified type.

        Examples
        --------
        Get a reference to the 2nd element when it was one of the types of Sensor:
        >>> entry_obj = obj.Sensors_element_as(2, models.Accelerometer)
        >>> entry_obj = obj.Sensors_element_as(2, models.Inclinometer)
        >>> entry_obj = obj.Sensors_element_as(2, models.LoadSensor)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = obj.Sensors_element_as(2, models.SensorInsert)
        """
        val = self.Sensors[index]
        if not isinstance(val, element_cls):
            raise TypeError(f"Expected value of type '{element_cls.__name__}' at index {index} but found type '{type(val).__name__}'")
        return val


    def _entity(self) -> bool:
        return True


CrossSection.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.first_order_response import FirstOrderResponse
from dnv_bladed_models.immediate_response import ImmediateResponse
from dnv_bladed_models.proportional_integral_deriviative import ProportionalIntegralDeriviative
from dnv_bladed_models.response_to_demand import ResponseToDemand
from dnv_bladed_models.response_to_demand_insert import ResponseToDemandInsert
from dnv_bladed_models.second_order_response import SecondOrderResponse
class PitchSystemDemand_PitchSystemDemandTypeEnum(str, Enum):
    POSITION = "Position"
    RATE = "Rate"
    INSERT = "Insert"

from .schema_helper import SchemaHelper
from .models_impl import *

TResponseToDemandOptions = TypeVar('TResponseToDemandOptions', FirstOrderResponse, ImmediateResponse, ResponseToDemandInsert, ProportionalIntegralDeriviative, SecondOrderResponse, ResponseToDemand, )

class PitchSystemDemand(BladedModel, ABC):
    r"""
    The common properties of the pitch and rate demand responses.
    
    Attributes
    ----------
    PitchSystemDemandType : PitchSystemDemand_PitchSystemDemandTypeEnum
        Defines the specific type of model in use.
    
    ResponseToDemand : Union[FirstOrderResponse, ImmediateResponse, ResponseToDemandInsert, ProportionalIntegralDeriviative, SecondOrderResponse]
    
    Notes
    -----
    
    This class is an abstraction, with the following concrete implementations:
        - PitchSystemDemandInsert
        - PitchPositionDemand
        - PitchRateDemand
    
    """
    PitchSystemDemandType: PitchSystemDemand_PitchSystemDemandTypeEnum = Field(alias="PitchSystemDemandType", default=None)
    ResponseToDemand: Union[FirstOrderResponse, ImmediateResponse, ResponseToDemandInsert, ProportionalIntegralDeriviative, SecondOrderResponse] = Field(alias="ResponseToDemand", default=None, discriminator='ResponseToDemandType')

    _type_info = TypeInfo(
        set([('ResponseToDemand', 'ResponseToDemandType'),]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    @property
    def ResponseToDemand_as_FirstOrderResponse(self) -> FirstOrderResponse:
        """
        Retrieves the value of ResponseToDemand guaranteeing it is a FirstOrderResponse; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        FirstOrderResponse
            A model object, guaranteed to be a FirstOrderResponse.

        Raises
        ------
        TypeError
            If the value is not a FirstOrderResponse.
        """
        return self.ResponseToDemand_as(FirstOrderResponse)


    @property
    def ResponseToDemand_as_ImmediateResponse(self) -> ImmediateResponse:
        """
        Retrieves the value of ResponseToDemand guaranteeing it is a ImmediateResponse; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        ImmediateResponse
            A model object, guaranteed to be a ImmediateResponse.

        Raises
        ------
        TypeError
            If the value is not a ImmediateResponse.
        """
        return self.ResponseToDemand_as(ImmediateResponse)


    @property
    def ResponseToDemand_as_ProportionalIntegralDeriviative(self) -> ProportionalIntegralDeriviative:
        """
        Retrieves the value of ResponseToDemand guaranteeing it is a ProportionalIntegralDeriviative; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        ProportionalIntegralDeriviative
            A model object, guaranteed to be a ProportionalIntegralDeriviative.

        Raises
        ------
        TypeError
            If the value is not a ProportionalIntegralDeriviative.
        """
        return self.ResponseToDemand_as(ProportionalIntegralDeriviative)


    @property
    def ResponseToDemand_as_SecondOrderResponse(self) -> SecondOrderResponse:
        """
        Retrieves the value of ResponseToDemand guaranteeing it is a SecondOrderResponse; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        SecondOrderResponse
            A model object, guaranteed to be a SecondOrderResponse.

        Raises
        ------
        TypeError
            If the value is not a SecondOrderResponse.
        """
        return self.ResponseToDemand_as(SecondOrderResponse)


    @property
    def ResponseToDemand_as_inline(self) -> Union[FirstOrderResponse, ImmediateResponse, ProportionalIntegralDeriviative, SecondOrderResponse]:
        """
        Retrieves the value of ResponseToDemand as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[FirstOrderResponse, ImmediateResponse, ProportionalIntegralDeriviative, SecondOrderResponse]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of ResponseToDemand; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.ResponseToDemand, ResponseToDemandInsert) or self.ResponseToDemand.is_insert:
            raise TypeError(f"Expected ResponseToDemand value to be an in-line object, but it is currently in the '$insert' state.")
        return self.ResponseToDemand


    def ResponseToDemand_as(self, cls: Type[TResponseToDemandOptions])-> TResponseToDemandOptions:
        """
        Retrieves the value of ResponseToDemand, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of ResponseToDemand, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[FirstOrderResponse, ImmediateResponse, ResponseToDemandInsert, ProportionalIntegralDeriviative, SecondOrderResponse]]
            One of the valid concrete types of ResponseToDemand, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TResponseToDemandOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of ResponseToDemand:
        >>> val_obj = model_obj.ResponseToDemand_as(models.FirstOrderResponse)
        >>> val_obj = model_obj.ResponseToDemand_as(models.ImmediateResponse)
        >>> val_obj = model_obj.ResponseToDemand_as(models.ProportionalIntegralDeriviative)
        >>> val_obj = model_obj.ResponseToDemand_as(models.SecondOrderResponse)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.ResponseToDemand_as(models.ResponseToDemandInsert)
        """
        if not isinstance(self.ResponseToDemand, cls):
            raise TypeError(f"Expected ResponseToDemand of type '{cls.__name__}' but was type '{type(self.ResponseToDemand).__name__}'")
        return self.ResponseToDemand


PitchSystemDemand.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.time_vs_wind_mean_speed import TimeVsWindMeanSpeed
from dnv_bladed_models.wind_mean_speed_variation import WindMeanSpeedVariation

from .schema_helper import SchemaHelper
from .models_impl import *


class WindMeanSpeedTimeHistory(WindMeanSpeedVariation):
    r"""
    A time history of the wind speed.
    
    Attributes
    ----------
    MeanSpeedVariationType : Literal['TimeHistory'], default='TimeHistory'
        Defines the specific type of MeanSpeedVariation model in use.  For a `TimeHistory` object, this must always be set to a value of `TimeHistory`.
    
    TimeVsMeanSpeed : List[TimeVsWindMeanSpeed]
        A list of points specifying the wind speed at the corresponding time.  The wind speed will be interpolated between these points; the wind speed before the first point will be constant at the first point's value; and the wind speed after the last point will remain constant at the last point's value
    
    Notes
    -----
    
    """
    MeanSpeedVariationType: Literal['TimeHistory'] = Field(alias="MeanSpeedVariationType", default='TimeHistory', allow_mutation=False, const=True) # type: ignore
    TimeVsMeanSpeed: List[TimeVsWindMeanSpeed] = Field(alias="TimeVsMeanSpeed", default=list())

    _relative_schema_path = 'TimeDomainSimulation/Environment/Wind/WindMeanSpeedVariation/WindMeanSpeedTimeHistory.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['TimeVsMeanSpeed',]),
        'MeanSpeedVariationType').merge(WindMeanSpeedVariation._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


WindMeanSpeedTimeHistory.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.current_direction_shear_variation import CurrentDirectionShearVariation
from dnv_bladed_models.time_vs_current_direction_shear import TimeVsCurrentDirectionShear

from .schema_helper import SchemaHelper
from .models_impl import *


class CurrentDirectionShearTimeHistory(CurrentDirectionShearVariation):
    r"""
    A time history of the vertical direction shear (veer).  This is a linear relationship between the height and the local direction, with the direction being its nominal value at the reference height.
    
    Not supported yet.
    
    Attributes
    ----------
    DirectionShearVariationType : Literal['TimeHistory'], default='TimeHistory', Not supported yet
        Defines the specific type of DirectionShearVariation model in use.  For a `TimeHistory` object, this must always be set to a value of `TimeHistory`.
    
    TimeVsDirectionShear : List[TimeVsCurrentDirectionShear], Not supported yet
        A list of points specifying the wind veer at the corresponding time.  The wind veer will be interpolated between these points; the wind veer before the first point will be constant at the first point's value; and the wind veer after the last point will remain constant at the last point's value
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    DirectionShearVariationType: Literal['TimeHistory'] = Field(alias="DirectionShearVariationType", default='TimeHistory', allow_mutation=False, const=True) # Not supported yet # type: ignore
    TimeVsDirectionShear: List[TimeVsCurrentDirectionShear] = Field(alias="TimeVsDirectionShear", default=list()) # Not supported yet

    _relative_schema_path = 'TimeDomainSimulation/Environment/SeaState/Current/CurrentDirectionShearVariation/CurrentDirectionShearTimeHistory.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['TimeVsDirectionShear',]),
        'DirectionShearVariationType').merge(CurrentDirectionShearVariation._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


CurrentDirectionShearTimeHistory.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.lidar_scanning_pattern import LidarScanningPattern
class PresetLidarScan_BeamSamplingEnum(str, Enum):
    SIMULTANEOUS = "Simultaneous"
    SEQUENTIAL = "Sequential"

from .schema_helper import SchemaHelper
from .models_impl import *


class PresetLidarScan(LidarScanningPattern, ABC):
    r"""
    the common properties of scan patterns which are set by the Lidar, rather than controlled in realtime by the turbine's controller.
    
    Not supported yet.
    
    Attributes
    ----------
    SamplesPerScan : int, default=1, Not supported yet
        The number of samples taken during a complete scan.
    
    LidarInterval : float, Not supported yet
        The time interval between scans.  Note that all fossing distances will be sampled at the same time.
    
    BeamSampling : PresetLidarScan_BeamSamplingEnum, Not supported yet
        The method of how each beam is sampled - whether is is simultaneously, or in sequence.
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    SamplesPerScan: int = Field(alias="SamplesPerScan", default=None) # Not supported yet
    LidarInterval: float = Field(alias="LidarInterval", default=None) # Not supported yet
    BeamSampling: PresetLidarScan_BeamSamplingEnum = Field(alias="BeamSampling", default=None) # Not supported yet

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(LidarScanningPattern._type_info)


PresetLidarScan.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
class Foundation_FoundationTypeEnum(str, Enum):
    LINEAR_FOUNDATION = "LinearFoundation"
    SIMPLIFIED_LINEAR_FOUNDATION = "SimplifiedLinearFoundation"
    INSERT = "Insert"

from .schema_helper import SchemaHelper
from .models_impl import *


class Foundation(BladedModel, ABC):
    r"""
    FOUNDATION\"
    
    Attributes
    ----------
    FoundationType : Foundation_FoundationTypeEnum
        Defines the specific type of model in use.
    
    UseFiniteElementDeflectionsForFoundationLoads : bool, default=False
        When this feature is enabled, support structure deflections calculated from the underlying finite element model are used to evaluate the foundation applied loads. The effect of these foundation loads is included when evaluating the turbine dynamic response.  This model is only active for time domain simulations. The foundation applied loads are calculated from the finite element deflections from the previous time step. This means that time step convergence studies may be required to ensure the accuracy of this model.  When this feature is enabled, the support structure node deflection outputs will be based on the finite element model rather than the modal deflections that are normally used. This ensures consistency between the foundation applied loads and the support structure node deflections.
    
    Notes
    -----
    
    This class is an abstraction, with the following concrete implementations:
        - FoundationInsert
        - LinearFoundation
        - SimplifiedLinearFoundation
    
    """
    FoundationType: Foundation_FoundationTypeEnum = Field(alias="FoundationType", default=None)
    UseFiniteElementDeflectionsForFoundationLoads: bool = Field(alias="UseFiniteElementDeflectionsForFoundationLoads", default=None)

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


Foundation.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.actuator_response import ActuatorResponse
from dnv_bladed_models.actuator_response_insert import ActuatorResponseInsert
from dnv_bladed_models.first_order_actuator_response import FirstOrderActuatorResponse
from dnv_bladed_models.instantaneous_actuator_response import InstantaneousActuatorResponse
from dnv_bladed_models.pitch_actuator import PitchActuator
from dnv_bladed_models.pitch_force_limits import PitchForceLimits
from dnv_bladed_models.second_order_actuator_response import SecondOrderActuatorResponse

from .schema_helper import SchemaHelper
from .models_impl import *

TActuatorResponseOptions = TypeVar('TActuatorResponseOptions', FirstOrderActuatorResponse, ActuatorResponseInsert, InstantaneousActuatorResponse, SecondOrderActuatorResponse, ActuatorResponse, )

class LinearPitchActuator(PitchActuator):
    r"""
    A linear pitch actuation definition.
    
    Attributes
    ----------
    ActuatorDriveType : Literal['LinearActuator'], default='LinearActuator'
        Defines the specific type of ActuatorDrive model in use.  For a `LinearActuator` object, this must always be set to a value of `LinearActuator`.
    
    ForceResponse : Union[FirstOrderActuatorResponse, ActuatorResponseInsert, InstantaneousActuatorResponse, SecondOrderActuatorResponse]
    
    PivotOffset : float
        The distance between the pitch axis and the pivot centre for the ram.
    
    PitchAngleAtMaximumTorque : float
        The pitch angle when the ram is perpendicular to the moment arm radius.
    
    BrakeForce : float, default=0
        The maximum restraining brake force applied when the safety limit switches are tripped or permanently on in idling and parked simulations.
    
    ForceLimits : PitchForceLimits
    
    BackupPowerForceLimits : PitchForceLimits
    
    RadiusOfArm : float
        The distance between the pitch axis and the connection point of the ram.
    
    Notes
    -----
    
    """
    ActuatorDriveType: Literal['LinearActuator'] = Field(alias="ActuatorDriveType", default='LinearActuator', allow_mutation=False, const=True) # type: ignore
    ForceResponse: Union[FirstOrderActuatorResponse, ActuatorResponseInsert, InstantaneousActuatorResponse, SecondOrderActuatorResponse] = Field(alias="ForceResponse", default=None, discriminator='ActuatorResponseType')
    PivotOffset: float = Field(alias="PivotOffset", default=None)
    PitchAngleAtMaximumTorque: float = Field(alias="PitchAngleAtMaximumTorque", default=None)
    BrakeForce: float = Field(alias="BrakeForce", default=None)
    ForceLimits: PitchForceLimits = Field(alias="ForceLimits", default=None)
    BackupPowerForceLimits: PitchForceLimits = Field(alias="BackupPowerForceLimits", default=None)
    RadiusOfArm: float = Field(alias="RadiusOfArm", default=None)

    _relative_schema_path = 'Components/PitchSystem/PitchActuator/LinearPitchActuator.json'
    _type_info = TypeInfo(
        set([('ForceResponse', 'ActuatorResponseType'),]),
        set([]),
        set([]),
        'ActuatorDriveType').merge(PitchActuator._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def ForceResponse_as_FirstOrderActuatorResponse(self) -> FirstOrderActuatorResponse:
        """
        Retrieves the value of ForceResponse guaranteeing it is a FirstOrderActuatorResponse; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        FirstOrderActuatorResponse
            A model object, guaranteed to be a FirstOrderActuatorResponse.

        Raises
        ------
        TypeError
            If the value is not a FirstOrderActuatorResponse.
        """
        return self.ForceResponse_as(FirstOrderActuatorResponse)


    @property
    def ForceResponse_as_InstantaneousActuatorResponse(self) -> InstantaneousActuatorResponse:
        """
        Retrieves the value of ForceResponse guaranteeing it is a InstantaneousActuatorResponse; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        InstantaneousActuatorResponse
            A model object, guaranteed to be a InstantaneousActuatorResponse.

        Raises
        ------
        TypeError
            If the value is not a InstantaneousActuatorResponse.
        """
        return self.ForceResponse_as(InstantaneousActuatorResponse)


    @property
    def ForceResponse_as_SecondOrderActuatorResponse(self) -> SecondOrderActuatorResponse:
        """
        Retrieves the value of ForceResponse guaranteeing it is a SecondOrderActuatorResponse; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        SecondOrderActuatorResponse
            A model object, guaranteed to be a SecondOrderActuatorResponse.

        Raises
        ------
        TypeError
            If the value is not a SecondOrderActuatorResponse.
        """
        return self.ForceResponse_as(SecondOrderActuatorResponse)


    @property
    def ForceResponse_as_inline(self) -> Union[FirstOrderActuatorResponse, InstantaneousActuatorResponse, SecondOrderActuatorResponse]:
        """
        Retrieves the value of ForceResponse as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[FirstOrderActuatorResponse, InstantaneousActuatorResponse, SecondOrderActuatorResponse]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of ActuatorResponse; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.ForceResponse, ActuatorResponseInsert) or self.ForceResponse.is_insert:
            raise TypeError(f"Expected ForceResponse value to be an in-line object, but it is currently in the '$insert' state.")
        return self.ForceResponse


    def ForceResponse_as(self, cls: Type[TActuatorResponseOptions])-> TActuatorResponseOptions:
        """
        Retrieves the value of ForceResponse, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of ActuatorResponse, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[FirstOrderActuatorResponse, ActuatorResponseInsert, InstantaneousActuatorResponse, SecondOrderActuatorResponse]]
            One of the valid concrete types of ActuatorResponse, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TActuatorResponseOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of ActuatorResponse:
        >>> val_obj = model_obj.ForceResponse_as(models.FirstOrderActuatorResponse)
        >>> val_obj = model_obj.ForceResponse_as(models.InstantaneousActuatorResponse)
        >>> val_obj = model_obj.ForceResponse_as(models.SecondOrderActuatorResponse)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.ForceResponse_as(models.ActuatorResponseInsert)
        """
        if not isinstance(self.ForceResponse, cls):
            raise TypeError(f"Expected ForceResponse of type '{cls.__name__}' but was type '{type(self.ForceResponse).__name__}'")
        return self.ForceResponse


    def _entity(self) -> bool:
        return True


LinearPitchActuator.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.added_inertia import AddedInertia
from dnv_bladed_models.added_inertia_insert import AddedInertiaInsert
from dnv_bladed_models.explicit_tower_can import ExplicitTowerCan
from dnv_bladed_models.foundation import Foundation
from dnv_bladed_models.foundation_insert import FoundationInsert
from dnv_bladed_models.linear_foundation import LinearFoundation
from dnv_bladed_models.point_inertia import PointInertia
from dnv_bladed_models.simple_tower_can import SimpleTowerCan
from dnv_bladed_models.simplified_linear_foundation import SimplifiedLinearFoundation
from dnv_bladed_models.six_by_six_inertia import SixBySixInertia
from dnv_bladed_models.structure import Structure
from dnv_bladed_models.tower_aerodynamic_properties import TowerAerodynamicProperties
from dnv_bladed_models.tower_can import TowerCan
from dnv_bladed_models.tower_can_insert import TowerCanInsert
from dnv_bladed_models.tower_connectable_nodes import TowerConnectableNodes
from dnv_bladed_models.tower_hydrodynamic_properties import TowerHydrodynamicProperties
from dnv_bladed_models.tower_materials_library import TowerMaterialsLibrary
from dnv_bladed_models.tower_output_group_library import TowerOutputGroupLibrary

from .schema_helper import SchemaHelper
from .models_impl import *

TFoundationOptions = TypeVar('TFoundationOptions', FoundationInsert, LinearFoundation, SimplifiedLinearFoundation, Foundation, )
TTowerCanOptions = TypeVar('TTowerCanOptions', ExplicitTowerCan, TowerCanInsert, SimpleTowerCan, TowerCan, )
TAddedInertiaOptions = TypeVar('TAddedInertiaOptions', AddedInertiaInsert, PointInertia, SixBySixInertia, AddedInertia, )

class Tower(Structure):
    r"""
    An axisymmetric tower, made from a series of tower \"cans\".
    
    Attributes
    ----------
    ComponentType : Literal['Tower'], default='Tower'
        Defines the specific type of Component model in use.  For a `Tower` object, this must always be set to a value of `Tower`.
    
    MaterialsLibrary : TowerMaterialsLibrary
    
    Foundation : Union[FoundationInsert, LinearFoundation, SimplifiedLinearFoundation]
    
    Cans : List[Union[ExplicitTowerCan, TowerCanInsert, SimpleTowerCan]]
        A list of cans, each one placed on top of the previous one.  These cans can be either prismatic or tapered.
    
    AerodynamicProperties : TowerAerodynamicProperties
    
    HydrodynamicProperties : TowerHydrodynamicProperties, Not supported yet
    
    PointInertias : List[Union[AddedInertiaInsert, PointInertia, SixBySixInertia]]
        A list of additional inertias to add to the tower.
    
    OutputGroups : TowerOutputGroupLibrary, Not supported yet
    
    ConnectableNodes : TowerConnectableNodes, Not supported yet
    
    Notes
    -----
    
    """
    ComponentType: Literal['Tower'] = Field(alias="ComponentType", default='Tower', allow_mutation=False, const=True) # type: ignore
    MaterialsLibrary: TowerMaterialsLibrary = Field(alias="MaterialsLibrary", default=TowerMaterialsLibrary())
    Foundation: Union[FoundationInsert, LinearFoundation, SimplifiedLinearFoundation] = Field(alias="Foundation", default=None, discriminator='FoundationType')
    Cans: List[Annotated[Union[ExplicitTowerCan, TowerCanInsert, SimpleTowerCan], Field(discriminator='TowerCanType')]] = Field(alias="Cans", default=list())
    AerodynamicProperties: TowerAerodynamicProperties = Field(alias="AerodynamicProperties", default=None)
    HydrodynamicProperties: TowerHydrodynamicProperties = Field(alias="HydrodynamicProperties", default=None) # Not supported yet
    PointInertias: List[Annotated[Union[AddedInertiaInsert, PointInertia, SixBySixInertia], Field(discriminator='AddedInertiaType')]] = Field(alias="PointInertias", default=list())
    OutputGroups: TowerOutputGroupLibrary = Field(alias="OutputGroups", default=TowerOutputGroupLibrary()) # Not supported yet
    ConnectableNodes: TowerConnectableNodes = Field(alias="ConnectableNodes", default=TowerConnectableNodes()) # Not supported yet

    _relative_schema_path = 'Components/Tower/Tower.json'
    _type_info = TypeInfo(
        set([('Foundation', 'FoundationType'),]),
        set([('Cans', 'TowerCanType'),('PointInertias', 'AddedInertiaType'),]),
        set(['MaterialsLibrary','Cans','PointInertias','OutputGroups','ConnectableNodes',]),
        'ComponentType').merge(Structure._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def Foundation_as_LinearFoundation(self) -> LinearFoundation:
        """
        Retrieves the value of Foundation guaranteeing it is a LinearFoundation; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        LinearFoundation
            A model object, guaranteed to be a LinearFoundation.

        Raises
        ------
        TypeError
            If the value is not a LinearFoundation.
        """
        return self.Foundation_as(LinearFoundation)


    @property
    def Foundation_as_SimplifiedLinearFoundation(self) -> SimplifiedLinearFoundation:
        """
        Retrieves the value of Foundation guaranteeing it is a SimplifiedLinearFoundation; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        SimplifiedLinearFoundation
            A model object, guaranteed to be a SimplifiedLinearFoundation.

        Raises
        ------
        TypeError
            If the value is not a SimplifiedLinearFoundation.
        """
        return self.Foundation_as(SimplifiedLinearFoundation)


    @property
    def Foundation_as_inline(self) -> Union[LinearFoundation, SimplifiedLinearFoundation]:
        """
        Retrieves the value of Foundation as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[LinearFoundation, SimplifiedLinearFoundation]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of Foundation; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.Foundation, FoundationInsert) or self.Foundation.is_insert:
            raise TypeError(f"Expected Foundation value to be an in-line object, but it is currently in the '$insert' state.")
        return self.Foundation


    def Foundation_as(self, cls: Type[TFoundationOptions])-> TFoundationOptions:
        """
        Retrieves the value of Foundation, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of Foundation, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[FoundationInsert, LinearFoundation, SimplifiedLinearFoundation]]
            One of the valid concrete types of Foundation, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TFoundationOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of Foundation:
        >>> val_obj = model_obj.Foundation_as(models.LinearFoundation)
        >>> val_obj = model_obj.Foundation_as(models.SimplifiedLinearFoundation)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.Foundation_as(models.FoundationInsert)
        """
        if not isinstance(self.Foundation, cls):
            raise TypeError(f"Expected Foundation of type '{cls.__name__}' but was type '{type(self.Foundation).__name__}'")
        return self.Foundation


    @property
    def Cans_as_inline(self) -> Iterable[Union[ExplicitTowerCan, SimpleTowerCan]]:
        """
        Retrieves the value of Cans as an iterable of model objects; if any of the values are specified with a '$insert', an error is raised.

        Returns
        -------
        Iterable[Union[ExplicitTowerCan, SimpleTowerCan]]
            A list of model objects, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If any of the values are not one of the concrete types of TowerCan; i.e. they are specified with an 'insert'.

        Examples
        --------
        >>> for i, TowerCan_obj in enumerate(obj.Cans_as_any):
                print(f"Element {i} is of type TowerCan_obj.TowerCanType}")

        or

        >>> for TowerCan_obj in obj.Cans_as_any:
                # process object
        """
        for val in self.Cans:
            if isinstance(val, TowerCanInsert) or val.is_insert == True:
                raise TypeError(f"Expected element value to be an in-line object, but it is currently in the '$insert' state.")
            yield val


    def Cans_element_as_ExplicitTowerCan(self, index: int) -> ExplicitTowerCan:
        """
        Retrieves an object from the Cans array field, ensuring it is a ExplicitTowerCan.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        ExplicitTowerCan
            A model object at the specified index, guaranteed to be a ExplicitTowerCan.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a ExplicitTowerCan.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a ExplicitTowerCan, that is specified in-line:
        >>> entry_obj = obj.Cans_element_as_ExplicitTowerCan(2)
        """
        return self.Cans_element_as(index, ExplicitTowerCan)


    def Cans_element_as_SimpleTowerCan(self, index: int) -> SimpleTowerCan:
        """
        Retrieves an object from the Cans array field, ensuring it is a SimpleTowerCan.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        SimpleTowerCan
            A model object at the specified index, guaranteed to be a SimpleTowerCan.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a SimpleTowerCan.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a SimpleTowerCan, that is specified in-line:
        >>> entry_obj = obj.Cans_element_as_SimpleTowerCan(2)
        """
        return self.Cans_element_as(index, SimpleTowerCan)


    def Cans_element_as_inline(self, index: int) -> Union[ExplicitTowerCan, SimpleTowerCan]:
        """
        Retrieves an object from the Cans array field; if the value is specified with a '$insert', an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        Union[ExplicitTowerCan, SimpleTowerCan]
            A model object at the specified index, guaranteed to not be an '$insert'.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not one of the concrete types of TowerCan; i.e. it is specified with an '$insert'.

        Examples
        --------
        Get a reference to the 2nd element, as any of the concrete types:
        >>> Cans_obj = obj.Cans_element_as_inline(2)
        """
        val = self.Cans[index]
        if isinstance(val, TowerCanInsert) or val.is_insert == True:
            raise TypeError(f"Expected element value at '{index}' to be an in-line object, but it is currently in the '$insert' state.")
        return val


    def Cans_element_as(self, index: int, element_cls: Type[TTowerCanOptions]) -> TTowerCanOptions:
        """
        Retrieves an object from the Cans array field, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of TowerCan, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        element_cls: Type[Union[ExplicitTowerCan, TowerCanInsert, SimpleTowerCan]]
            One of the valid concrete types of TowerCan, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TTowerCanOptions
            A model object of the specified type.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the model object for the specified index is not of the specified type.

        Examples
        --------
        Get a reference to the 2nd element when it was one of the types of TowerCan:
        >>> entry_obj = obj.Cans_element_as(2, models.ExplicitTowerCan)
        >>> entry_obj = obj.Cans_element_as(2, models.SimpleTowerCan)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = obj.Cans_element_as(2, models.TowerCanInsert)
        """
        val = self.Cans[index]
        if not isinstance(val, element_cls):
            raise TypeError(f"Expected value of type '{element_cls.__name__}' at index {index} but found type '{type(val).__name__}'")
        return val


    @property
    def PointInertias_as_inline(self) -> Iterable[Union[PointInertia, SixBySixInertia]]:
        """
        Retrieves the value of PointInertias as an iterable of model objects; if any of the values are specified with a '$insert', an error is raised.

        Returns
        -------
        Iterable[Union[PointInertia, SixBySixInertia]]
            A list of model objects, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If any of the values are not one of the concrete types of AddedInertia; i.e. they are specified with an 'insert'.

        Examples
        --------
        >>> for i, AddedInertia_obj in enumerate(obj.PointInertias_as_any):
                print(f"Element {i} is of type AddedInertia_obj.AddedInertiaType}")

        or

        >>> for AddedInertia_obj in obj.PointInertias_as_any:
                # process object
        """
        for val in self.PointInertias:
            if isinstance(val, AddedInertiaInsert) or val.is_insert == True:
                raise TypeError(f"Expected element value to be an in-line object, but it is currently in the '$insert' state.")
            yield val


    def PointInertias_element_as_PointInertia(self, index: int) -> PointInertia:
        """
        Retrieves an object from the PointInertias array field, ensuring it is a PointInertia.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        PointInertia
            A model object at the specified index, guaranteed to be a PointInertia.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a PointInertia.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a PointInertia, that is specified in-line:
        >>> entry_obj = obj.PointInertias_element_as_PointInertia(2)
        """
        return self.PointInertias_element_as(index, PointInertia)


    def PointInertias_element_as_SixBySixInertia(self, index: int) -> SixBySixInertia:
        """
        Retrieves an object from the PointInertias array field, ensuring it is a SixBySixInertia.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        SixBySixInertia
            A model object at the specified index, guaranteed to be a SixBySixInertia.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a SixBySixInertia.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a SixBySixInertia, that is specified in-line:
        >>> entry_obj = obj.PointInertias_element_as_SixBySixInertia(2)
        """
        return self.PointInertias_element_as(index, SixBySixInertia)


    def PointInertias_element_as_inline(self, index: int) -> Union[PointInertia, SixBySixInertia]:
        """
        Retrieves an object from the PointInertias array field; if the value is specified with a '$insert', an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        Union[PointInertia, SixBySixInertia]
            A model object at the specified index, guaranteed to not be an '$insert'.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not one of the concrete types of AddedInertia; i.e. it is specified with an '$insert'.

        Examples
        --------
        Get a reference to the 2nd element, as any of the concrete types:
        >>> PointInertias_obj = obj.PointInertias_element_as_inline(2)
        """
        val = self.PointInertias[index]
        if isinstance(val, AddedInertiaInsert) or val.is_insert == True:
            raise TypeError(f"Expected element value at '{index}' to be an in-line object, but it is currently in the '$insert' state.")
        return val


    def PointInertias_element_as(self, index: int, element_cls: Type[TAddedInertiaOptions]) -> TAddedInertiaOptions:
        """
        Retrieves an object from the PointInertias array field, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of AddedInertia, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        element_cls: Type[Union[AddedInertiaInsert, PointInertia, SixBySixInertia]]
            One of the valid concrete types of AddedInertia, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TAddedInertiaOptions
            A model object of the specified type.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the model object for the specified index is not of the specified type.

        Examples
        --------
        Get a reference to the 2nd element when it was one of the types of AddedInertia:
        >>> entry_obj = obj.PointInertias_element_as(2, models.PointInertia)
        >>> entry_obj = obj.PointInertias_element_as(2, models.SixBySixInertia)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = obj.PointInertias_element_as(2, models.AddedInertiaInsert)
        """
        val = self.PointInertias[index]
        if not isinstance(val, element_cls):
            raise TypeError(f"Expected value of type '{element_cls.__name__}' at index {index} but found type '{type(val).__name__}'")
        return val


    def _entity(self) -> bool:
        return True


Tower.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.timed_event import TimedEvent

from .schema_helper import SchemaHelper
from .models_impl import *


class Fault(TimedEvent, ABC):
    r"""
    A fault which may occur during the simulation.  These can either be timed events, or based on a set of conditions which may or may not occur.
    
    Attributes
    ----------
    OnComponentInAssembly : str, regex=^Assembly.(.+)$
        A qualified, dot-separated path to a component in the assembly tree to which this applies.  i.e. `Assembly.<name1>.<name2>`
    
    Notes
    -----
    
    """
    OnComponentInAssembly: str = Field(alias="@OnComponentInAssembly", default=None, regex='^Assembly.(.+)$')

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(TimedEvent._type_info)


Fault.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel

from .schema_helper import SchemaHelper
from .models_impl import *


class BladeLoadOutputs(BladedModel):
    r"""
    Loads selected
    
    Not supported yet.
    
    Attributes
    ----------
    FlapwiseBendingLoads : bool, default=False, Not supported yet
        Output blade bending moments for the flapwise direction (0=no, 1=yes).
    
    EdgewiseBendingLoads : bool, default=False, Not supported yet
        Output blade bending moments for the edgewise direction (0=no, 1=yes).
    
    FlapwiseShearLoads : bool, default=False, Not supported yet
        Output blade shear forces for the flapwise direction (0=no, 1=yes).
    
    EdgewiseShearLoads : bool, default=False, Not supported yet
        Output blade shear forces for the edgewise direction (0=no, 1=yes).
    
    OutOfPlaneBendingLoads : bool, default=False, Not supported yet
        Output blade bending moments for out of plane direction (0=no, 1=yes).
    
    InPlaneBendingLoads : bool, default=False, Not supported yet
        Output blade bending moments for in plane direction (0=no, 1=yes).
    
    OutOfPlaneShearLoads : bool, default=False, Not supported yet
        Output blade shear forces for out of plane direction (0=no, 1=yes).
    
    InPlaneShearLoads : bool, default=False, Not supported yet
        Output blade shear forces for in plane direction (0=no, 1=yes).
    
    RadialForces : bool, default=False, Not supported yet
        Output blade radial forces (0=no, 1=yes).
    
    LoadsInRootAxisSystem : bool, default=False, Not supported yet
        Output blade loads about the root axes system (0=no, 1=yes).
    
    LoadsInAeroAxisSystem : bool, default=False, Not supported yet
        Output blade loads about the aero axes system (0=no, 1=yes).
    
    LoadsInUserAxisSystem : bool, default=False, Not supported yet
        Output blade loads about the user defined axes system (0=no, 1=yes).
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    FlapwiseBendingLoads: bool = Field(alias="FlapwiseBendingLoads", default=None) # Not supported yet
    EdgewiseBendingLoads: bool = Field(alias="EdgewiseBendingLoads", default=None) # Not supported yet
    FlapwiseShearLoads: bool = Field(alias="FlapwiseShearLoads", default=None) # Not supported yet
    EdgewiseShearLoads: bool = Field(alias="EdgewiseShearLoads", default=None) # Not supported yet
    OutOfPlaneBendingLoads: bool = Field(alias="OutOfPlaneBendingLoads", default=None) # Not supported yet
    InPlaneBendingLoads: bool = Field(alias="InPlaneBendingLoads", default=None) # Not supported yet
    OutOfPlaneShearLoads: bool = Field(alias="OutOfPlaneShearLoads", default=None) # Not supported yet
    InPlaneShearLoads: bool = Field(alias="InPlaneShearLoads", default=None) # Not supported yet
    RadialForces: bool = Field(alias="RadialForces", default=None) # Not supported yet
    LoadsInRootAxisSystem: bool = Field(alias="LoadsInRootAxisSystem", default=None) # Not supported yet
    LoadsInAeroAxisSystem: bool = Field(alias="LoadsInAeroAxisSystem", default=None) # Not supported yet
    LoadsInUserAxisSystem: bool = Field(alias="LoadsInUserAxisSystem", default=None) # Not supported yet

    _relative_schema_path = 'Components/Blade/BladeOutputGroupLibrary/BladeOutputGroup/BladeLoadOutputs/BladeLoadOutputs.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


BladeLoadOutputs.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.blade_aerodynamic_outputs import BladeAerodynamicOutputs
from dnv_bladed_models.blade_hydrodynamic_outputs import BladeHydrodynamicOutputs
from dnv_bladed_models.blade_load_outputs import BladeLoadOutputs
from dnv_bladed_models.blade_motion_outputs import BladeMotionOutputs
from dnv_bladed_models.blade_outputs_for_cross_section import BladeOutputsForCrossSection
from dnv_bladed_models.blade_outputs_for_location import BladeOutputsForLocation
from dnv_bladed_models.blade_outputs_for_location_insert import BladeOutputsForLocationInsert
from dnv_bladed_models.blade_outputs_for_position import BladeOutputsForPosition
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.xml_blade_outputs import XMLBladeOutputs

from .schema_helper import SchemaHelper
from .models_impl import *

TBladeOutputsForLocationOptions = TypeVar('TBladeOutputsForLocationOptions', BladeOutputsForLocationInsert, BladeOutputsForCrossSection, BladeOutputsForPosition, BladeOutputsForLocation, )

class BladeOutputGroup(BladedModel):
    r"""
    An output group definition for the blade.
    
    Not supported yet.
    
    Attributes
    ----------
    XML : XMLBladeOutputs, Not supported yet
    
    LoadOutputs : BladeLoadOutputs, Not supported yet
    
    MotionOutputs : BladeMotionOutputs, Not supported yet
    
    AerodynamicOutputs : BladeAerodynamicOutputs, Not supported yet
    
    HydrodynamicOutputs : BladeHydrodynamicOutputs, Not supported yet
    
    OutputLocations : List[Union[BladeOutputsForLocationInsert, BladeOutputsForCrossSection, BladeOutputsForPosition]], Not supported yet
        List of BladeOutputsForLocation, originally enumerated by Fortran.OUTPUT.AEROD_NSTS
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    XML: XMLBladeOutputs = Field(alias="XML", default=None) # Not supported yet
    LoadOutputs: BladeLoadOutputs = Field(alias="LoadOutputs", default=None) # Not supported yet
    MotionOutputs: BladeMotionOutputs = Field(alias="MotionOutputs", default=None) # Not supported yet
    AerodynamicOutputs: BladeAerodynamicOutputs = Field(alias="AerodynamicOutputs", default=None) # Not supported yet
    HydrodynamicOutputs: BladeHydrodynamicOutputs = Field(alias="HydrodynamicOutputs", default=None) # Not supported yet
    OutputLocations: List[Annotated[Union[BladeOutputsForLocationInsert, BladeOutputsForCrossSection, BladeOutputsForPosition], Field(discriminator='BladeOutputsForLocationType')]] = Field(alias="OutputLocations", default=list()) # Not supported yet

    _relative_schema_path = 'Components/Blade/BladeOutputGroupLibrary/BladeOutputGroup/BladeOutputGroup.json'
    _type_info = TypeInfo(
        set([]),
        set([('OutputLocations', 'BladeOutputsForLocationType'),]),
        set(['OutputLocations',]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def OutputLocations_as_inline(self) -> Iterable[Union[BladeOutputsForCrossSection, BladeOutputsForPosition]]:
        """
        Retrieves the value of OutputLocations as an iterable of model objects; if any of the values are specified with a '$insert', an error is raised.

        Returns
        -------
        Iterable[Union[BladeOutputsForCrossSection, BladeOutputsForPosition]]
            A list of model objects, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If any of the values are not one of the concrete types of BladeOutputsForLocation; i.e. they are specified with an 'insert'.

        Examples
        --------
        >>> for i, BladeOutputsForLocation_obj in enumerate(obj.OutputLocations_as_any):
                print(f"Element {i} is of type BladeOutputsForLocation_obj.BladeOutputsForLocationType}")

        or

        >>> for BladeOutputsForLocation_obj in obj.OutputLocations_as_any:
                # process object
        """
        for val in self.OutputLocations:
            if isinstance(val, BladeOutputsForLocationInsert) or val.is_insert == True:
                raise TypeError(f"Expected element value to be an in-line object, but it is currently in the '$insert' state.")
            yield val


    def OutputLocations_element_as_BladeOutputsForCrossSection(self, index: int) -> BladeOutputsForCrossSection:
        """
        Retrieves an object from the OutputLocations array field, ensuring it is a BladeOutputsForCrossSection.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        BladeOutputsForCrossSection
            A model object at the specified index, guaranteed to be a BladeOutputsForCrossSection.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a BladeOutputsForCrossSection.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a BladeOutputsForCrossSection, that is specified in-line:
        >>> entry_obj = obj.OutputLocations_element_as_BladeOutputsForCrossSection(2)
        """
        return self.OutputLocations_element_as(index, BladeOutputsForCrossSection)


    def OutputLocations_element_as_BladeOutputsForPosition(self, index: int) -> BladeOutputsForPosition:
        """
        Retrieves an object from the OutputLocations array field, ensuring it is a BladeOutputsForPosition.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        BladeOutputsForPosition
            A model object at the specified index, guaranteed to be a BladeOutputsForPosition.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a BladeOutputsForPosition.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a BladeOutputsForPosition, that is specified in-line:
        >>> entry_obj = obj.OutputLocations_element_as_BladeOutputsForPosition(2)
        """
        return self.OutputLocations_element_as(index, BladeOutputsForPosition)


    def OutputLocations_element_as_inline(self, index: int) -> Union[BladeOutputsForCrossSection, BladeOutputsForPosition]:
        """
        Retrieves an object from the OutputLocations array field; if the value is specified with a '$insert', an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        Union[BladeOutputsForCrossSection, BladeOutputsForPosition]
            A model object at the specified index, guaranteed to not be an '$insert'.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not one of the concrete types of BladeOutputsForLocation; i.e. it is specified with an '$insert'.

        Examples
        --------
        Get a reference to the 2nd element, as any of the concrete types:
        >>> OutputLocations_obj = obj.OutputLocations_element_as_inline(2)
        """
        val = self.OutputLocations[index]
        if isinstance(val, BladeOutputsForLocationInsert) or val.is_insert == True:
            raise TypeError(f"Expected element value at '{index}' to be an in-line object, but it is currently in the '$insert' state.")
        return val


    def OutputLocations_element_as(self, index: int, element_cls: Type[TBladeOutputsForLocationOptions]) -> TBladeOutputsForLocationOptions:
        """
        Retrieves an object from the OutputLocations array field, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of BladeOutputsForLocation, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        element_cls: Type[Union[BladeOutputsForLocationInsert, BladeOutputsForCrossSection, BladeOutputsForPosition]]
            One of the valid concrete types of BladeOutputsForLocation, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TBladeOutputsForLocationOptions
            A model object of the specified type.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the model object for the specified index is not of the specified type.

        Examples
        --------
        Get a reference to the 2nd element when it was one of the types of BladeOutputsForLocation:
        >>> entry_obj = obj.OutputLocations_element_as(2, models.BladeOutputsForCrossSection)
        >>> entry_obj = obj.OutputLocations_element_as(2, models.BladeOutputsForPosition)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = obj.OutputLocations_element_as(2, models.BladeOutputsForLocationInsert)
        """
        val = self.OutputLocations[index]
        if not isinstance(val, element_cls):
            raise TypeError(f"Expected value of type '{element_cls.__name__}' at index {index} but found type '{type(val).__name__}'")
        return val


    def _entity(self) -> bool:
        return True


BladeOutputGroup.update_forward_refs()
